@@ -24,7 +24,6 @@ from .. import utils
 from .. import document
 
 from .veuszdialog import VeuszDialog
-from . import dataeditdialog
 
 def _(text, disambiguation=None, context="HistogramDialog"):
     """Translate text."""
@@ -58,8 +57,7 @@ class ManualBinModel(qt4.QAbstractListModel):
                 return False
 
             self.thedata[ index.row() ] = val
-            self.emit( qt4.SIGNAL("dataChanged(const QModelIndex &,"
-                                  " const QModelIndex &)"), index, index)
+            self.dataChanged.emit(index, index)
             return True
         return False
 
@@ -76,15 +74,13 @@ class HistoDataDialog(VeuszDialog):
         validator = qt4.QRegExpValidator(regexp, self)
         self.minval.setValidator(validator)
         self.maxval.setValidator(validator)
-        self.connect( self.buttonBox.button(qt4.QDialogButtonBox.Apply),
-                      qt4.SIGNAL("clicked()"), self.applyClicked )
-        self.connect( self.buttonBox.button(qt4.QDialogButtonBox.Reset),
-                      qt4.SIGNAL('clicked()'), self.resetClicked )
-        self.connect( self.bingenerate, qt4.SIGNAL('clicked()'),
-                      self.generateManualBins )
-        self.connect( self.binadd, qt4.SIGNAL('clicked()'), self.addManualBins )
-        self.connect( self.binremove, qt4.SIGNAL('clicked()'),
-                      self.removeManualBins )
+        self.buttonBox.button(qt4.QDialogButtonBox.Apply).clicked.connect(
+            self.applyClicked )
+        self.buttonBox.button(qt4.QDialogButtonBox.Reset).clicked.connect(
+            self.resetClicked )
+        self.bingenerate.clicked.connect(self.generateManualBins)
+        self.binadd.clicked.connect(self.addManualBins)
+        self.binremove.clicked.connect(self.removeManualBins)
 
         self.bindata = []
         self.binmodel = ManualBinModel(self.bindata)
@@ -142,9 +138,13 @@ class HistoDataDialog(VeuszDialog):
             islog = dialog.logarithmic.isChecked()
             self.binparams = (numbins, minval, maxval, islog)
 
-            self.expr = dialog.indataset.currentText()
-            self.outdataset = dialog.outdataset.currentText()
-            self.outbins = dialog.outbins.currentText()
+            self.expr = dialog.indataset.currentText().strip()
+            self.outdataset = dialog.outdataset.currentText().strip()
+            self.outbins = dialog.outbins.currentText().strip()
+
+            if self.expr == self.outdataset or self.expr == self.outbins:
+                raise RuntimeError(_("Output datasets cannot be the same as input datasets"))
+
             self.method = dialog.methodGroup.getRadioChecked().objectName()
             self.manualbins = list( dialog.bindata )
             self.manualbins.sort()
@@ -232,8 +232,8 @@ class HistoDataDialog(VeuszDialog):
 
         # need to map backwards to get dataset names
         revds = dict( (a,b) for b,a in citems(self.document.data) )
-        self.outdataset.setEditText( revds[gen.valuedataset] )
-        self.outbins.setEditText( revds[gen.bindataset] )
+        self.outdataset.setEditText(revds.get(gen.valuedataset, ''))
+        self.outbins.setEditText(revds.get(gen.bindataset, ''))
 
         # if there are parameters
         if gen.binparams:
@@ -291,5 +291,3 @@ def recreateDataset(mainwindow, document, dataset, datasetname):
     mainwindow.showDialog(dialog)
     dialog.reEditDataset(dataset, datasetname)
 
-dataeditdialog.recreate_register[document.DatasetHistoValues] = recreateDataset
-dataeditdialog.recreate_register[document.DatasetHistoBins] = recreateDataset
