@@ -26,27 +26,36 @@ external programs.
 
 from __future__ import division, print_function
 import os.path
-import traceback
+import numpy as N
 
-from ..compat import citems, ckeys, cbasestr, cstr
+from ..compat import cbasestr
 from .. import qtall as qt4
 from .. import setting
 from .. import embed
 from .. import plugins
 from .. import utils
 
-from . import importparams
 from . import datasets
 from . import operations
 from . import dataset_histo
+from . import dataset_filtered
 from . import mime
 from . import export
 
+def _(text, disambiguation=None, context='CommandInterface'):
+    """Translate text."""
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
+
+def registerImportCommand(name, method):
+    """Add command to command interface."""
+    setattr(CommandInterface, name, method)
+    CommandInterface.safe_commands.append(name)
+
 class CommandInterface(qt4.QObject):
     """Class provides command interface."""
 
     # commands which are safe in any script
-    safe_commands = (
+    safe_commands = [
         'Action',
         'Add',
         'AddCustom',
@@ -54,18 +63,13 @@ class CommandInterface(qt4.QObject):
         'CloneWidget',
         'CreateHistogram',
         'DatasetPlugin',
+        'FilterDatasets',
         'Get',
         'GetChildren',
         'GetData',
         'GetDataType',
         'GetDatasets',
         'ImportFITSFile',
-        'ImportFile',
-        'ImportFile2D',
-        'ImportFileCSV',
-        'ImportFilePlugin',
-        'ImportString',
-        'ImportString2D',
         'List',
         'NodeChildren',
         'NodeType',
@@ -90,7 +94,7 @@ class CommandInterface(qt4.QObject):
         'TagDatasets',
         'To',
         'WidgetType',
-        )
+        ]
 
     # commands which can modify disk, etc
     unsafe_commands = (
@@ -108,8 +112,7 @@ class CommandInterface(qt4.QObject):
         self.verbose = False
         self.importpath = []
 
-        self.connect( self.document, qt4.SIGNAL("sigWiped"),
-                      self.slotWipedDoc )
+        self.document.sigWiped.connect(self.slotWipedDoc)
 
         self.Root = embed.WidgetNode(self, 'widget', '/')
 
@@ -154,7 +157,7 @@ class CommandInterface(qt4.QObject):
         w = self.document.applyOperation(op)
 
         if self.verbose:
-            print("Added a widget of type '%s' (%s)" % (type, w.userdescription))
+            print(_("Added a widget of type '%s' (%s)") % (type, w.userdescription))
 
         return w.name
 
@@ -256,8 +259,8 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print(('Constructed histogram of "%s", creating datasets'
-                   ' "%s" and "%s"') % (inexpr, outbinsds, outvalsds))
+            print(_('Constructed histogram of "%s", creating datasets'
+                    ' "%s" and "%s"') % (inexpr, outbinsds, outvalsds))
 
     def DatasetPlugin(self, pluginname, fields, datasetnames={}):
         """Use a dataset plugin.
@@ -282,7 +285,7 @@ class CommandInterface(qt4.QObject):
         outdatasets = self.document.applyOperation(op)
 
         if self.verbose:
-            print("Used dataset plugin %s to make datasets %s" % (
+            print(_("Used dataset plugin %s to make datasets %s") % (
                 pluginname, ', '.join(outdatasets)))
 
     def Remove(self, name):
@@ -291,7 +294,7 @@ class CommandInterface(qt4.QObject):
         op = operations.OperationWidgetDelete(w)
         self.document.applyOperation(op)
         if self.verbose:
-            print("Removed widget '%s'" % name)
+            print(_("Removed widget '%s'") % name)
 
     def RemoveCustom(self, name):
         """Removes a custom-defined constant, function or import."""
@@ -315,7 +318,7 @@ class CommandInterface(qt4.QObject):
                                                    where)
 
         if self.verbose:
-            print("Changed to widget '%s'" % self.currentwidget.path)
+            print(_("Changed to widget '%s'") % self.currentwidget.path)
 
     def List(self, where='.'):
         """List the contents of a widget, by default the current widget."""
@@ -324,7 +327,7 @@ class CommandInterface(qt4.QObject):
         children = widget.childnames
 
         if len(children) == 0:
-            print('%30s' % 'No children found')
+            print('%30s' % _('No children found'))
         else:
             # output format name, type
             for name in children:
@@ -343,7 +346,7 @@ class CommandInterface(qt4.QObject):
 
     def GetDatasets(self):
         """Return a list of names of datasets."""
-        return sorted(ckeys(self.document.data))
+        return sorted(self.document.data)
 
     def ResolveReference(self, setn):
         """If the setting is set to a reference, follow the chain of
@@ -359,10 +362,14 @@ class CommandInterface(qt4.QObject):
         else:
             return None
 
-    def Save(self, filename):
-        """Save the state to a file."""
-        f = open(filename, 'w')
-        self.document.saveToFile(f)
+    def Save(self, filename, mode='vsz'):
+        """Save the state to a file.
+
+        mode can be:
+         'vsz': standard veusz text format
+         'hdf5': HDF5 format
+        """
+        self.document.save(filename, mode)
 
     def Set(self, var, val):
         """Set the value of a setting."""
@@ -372,8 +379,7 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print(( "Set setting '%s' to %s" %
-                    (var, repr(pref.get())) ))
+            print( _("Set setting '%s' to %s") % (var, repr(pref.get())) )
 
     def SetToReference(self, var, val):
         """Set setting to a reference value."""
@@ -383,8 +389,7 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print(( "Set setting '%s' to %s" %
-                    (var, repr(pref.get())) ))
+            print( _( "Set setting '%s' to %s") % (var, repr(pref.get())) )
 
     def SetData(self, name, val, symerr=None, negerr=None, poserr=None):
         """Set dataset with name with values (and optionally errors)."""
@@ -394,11 +399,15 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
  
         if self.verbose:
-            print("Set dataset '%s':" % name)
-            print(" Values = %s" % str( data.data ))
-            print(" Symmetric errors = %s" % str( data.serr ))
-            print(" Negative errors = %s" % str( data.nerr ))
-            print(" Positive errors = %s" % str( data.perr ))
+            print(
+                _("Set dataset '%s':\n"
+                  " Values = %s\n"
+                  " Symmetric errors = %s\n"
+                  " Negative errors = %s\n"
+                  " Positive errors = %s") % (
+                      name, str(data.data), str(data.serr),
+                      str(data.nerr), str(data.perr))
+            )
 
     def SetDataDateTime(self, name, vals):
         """Set datetime dataset to be values given.
@@ -410,8 +419,11 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print("Set dataset '%s':" % name)
-            print(" Values = %s" % str(ds.data))
+            print(
+                _("Set dataset '%s':\n"
+                  " Values = %s") % (
+                      name, str(ds.data))
+            )
 
     def SetDataExpression(self, name, val, symerr=None, negerr=None, poserr=None,
                           linked=False, parametric=None):
@@ -438,14 +450,18 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
         
         if self.verbose:
-            print("Set dataset '%s' based on expression:" % name)
-            print(" Values = %s" % str( data.data ))
-            print(" Symmetric errors = %s" % str( data.serr ))
-            print(" Negative errors = %s" % str( data.nerr ))
-            print(" Positive errors = %s" % str( data.perr ))
+            print(
+                _("Set dataset '%s' based on expression:\n"
+                  " Values = %s\n"
+                  " Symmetric errors = %s\n"
+                  " Negative errors = %s\n"
+                  " Positive errors = %s") % (
+                      name, str(data.data), str(data.serr),
+                      str(data.nerr), str(data.perr))
+            )
             if parametric:
-                print(" Where t goes form %g:%g in %i steps" % parametric)
-            print(" linked to expression = %s" % repr(linked))
+                print(_(" Where t goes form %g:%g in %i steps") % parametric)
+            print(_(" linked to expression = %s") % repr(linked))
 
     def SetDataRange(self, name, numsteps, val, symerr=None, negerr=None,
                      poserr=None, linked=False):
@@ -463,12 +479,16 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print("Set dataset '%s' based on range:" % name)
-            print(" Number of steps = %i" % numsteps)
-            print(" Range of data = %s" % repr(val))
-            print(" Range of symmetric error = %s" % repr(symerr))
-            print(" Range of positive error = %s" % repr(poserr))
-            print(" Range of negative error = %s" % repr(negerr))
+            print(
+                _("Set dataset '%s' based on range:\n"
+                  " Number of steps = %i\n"
+                  " Range of data = %s\n"
+                  " Range of symmetric error = %s\n"
+                  " Range of positive error = %s\n"
+                  " Range of negative error = %s" % (
+                      name, numsteps, repr(val),
+                      repr(symerr), repr(poserr), repr(negerr)) )
+              )
 
     def SetData2DExpression(self, name, expr, linked=False):
         """Create a 2D dataset based on expressions
@@ -482,11 +502,14 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
 
         if self.verbose:
-            print("Set 2D dataset '%s' based on expressions" % name)
-            print(" expression = %s" % repr(expr))
-            print(" linked to expression = %s" % repr(linked))
-            print(" Made a dataset (%i x %i)" % (data.data.shape[0],
-                                                 data.data.shape[1]))
+            print(
+                _("Set 2D dataset '%s' based on expressions\n"
+                  " expression = %s\n"
+                  " linked to expression = %s\n"
+                  " Made a dataset (%i x %i)") % (
+                      name, repr(expr), repr(linked),
+                      data.data.shape[0], data.data.shape[1])
+            )
 
     def SetData2DExpressionXYZ(self, name, xexpr, yexpr, zexpr, linked=False):
         """Create a 2D dataset based on expressions in x, y and z
@@ -502,13 +525,18 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
 
         if self.verbose:
-            print("Set 2D dataset '%s' based on expressions" % name)
-            print(" X expression = %s" % repr(xexpr))
-            print(" Y expression = %s" % repr(yexpr))
-            print(" Z expression = %s" % repr(zexpr))
-            print(" linked to expression = %s" % repr(linked))
-            print(" Made a dataset (%i x %i)" % (data.data.shape[0],
-                                                 data.data.shape[1]))
+            print(
+                _("Made 2D dataset '%s' based on expressions:\n"
+                  " X expression = %s\n"
+                  " Y expression = %s\n"
+                  " Z expression = %s\n"
+                  " is linked to expression = %s\n"
+                  " Shape (%i x %i)") % (
+                      name,
+                      repr(xexpr), repr(yexpr), repr(zexpr),
+                      repr(linked),
+                      data.data.shape[0], data.data.shape[1])
+            )
 
     def SetData2DXYFunc(self, name, xstep, ystep, expr, linked=False):
         """Create a 2D dataset based on expressions of a range of x and y
@@ -524,23 +552,57 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
 
         if self.verbose:
-            print("Set 2D dataset '%s' based on function of x and y" % name)
-            print(" X steps = %s" % repr(xstep))
-            print(" Y steps = %s" % repr(ystep))
-            print(" Expression = %s" % repr(expr))
-            print(" linked to expression = %s" % repr(linked))
-            print(" Made a dataset (%i x %i)" % (data.data.shape[0],
-                                                 data.data.shape[1]))
+            print(
+                _("Set 2D dataset '%s' based on function of x and y\n"
+                  " X steps = %s\n"
+                  " Y steps = %s\n"
+                  " Expression = %s\n"
+                  " linked to expression = %s\n"
+                  " Made a dataset (%i x %i)") % (
+                      name, repr(xstep), repr(ystep),
+                      repr(expr), repr(linked),
+                      data.data.shape[0], data.data.shape[1])
+            )
 
-    def SetData2D(self, name, data, xrange=None, yrange=None):
-        """Create a 2D dataset."""
+    def SetData2D(self, name, data, xrange=None, yrange=None,
+                  xedge=None, yedge=None,
+                  xcent=None, ycent=None):
+        """Create a 2D dataset.
 
-        data = datasets.Dataset2D(data, xrange=xrange, yrange=yrange)
+        name: name of dataset
+        data: 2d array
+        xrange: optional tuple with X range of data (min, max)
+        yrange: optional tuple with Y range of data (min, max)
+        xedge: x values for grid (instead of rangex)
+        yedge: y values for grid (instead of rangey)
+        xcent: x values for pixel centres (instead of rangex)
+        ycent: y values for pixel centres (instead of rangey)
+        """
+
+        data = N.array(data)
+
+        if ( (xedge is not None and not utils.checkAscending(xedge)) or
+             (yedge is not None and not utils.checkAscending(yedge)) ):
+            raise ValueError("xedge and yedge must be ascending, if given")
+        if ( (xcent is not None and not utils.checkAscending(xcent)) or
+             (ycent is not None and not utils.checkAscending(ycent)) ):
+            raise ValueError("xcent and ycent must be ascending, if given")
+
+        if ( (xedge is not None and len(xedge) != data.shape[1]+1) or
+             (yedge is not None and len(yedge) != data.shape[0]+1) ):
+            raise ValueError("xedge and yedge lengths must be data shape+1")
+        if ( (xcent is not None and len(xcent) != data.shape[1]) or
+             (ycent is not None and len(ycent) != data.shape[0]) ):
+            raise ValueError("xcent and ycent lengths must be data shape")
+
+        data = datasets.Dataset2D(data, xrange=xrange, yrange=yrange,
+                                  xedge=xedge, yedge=yedge,
+                                  xcent=xcent, ycent=ycent)
         op = operations.OperationDatasetSet(name, data)
         self.document.applyOperation(op)
 
         if self.verbose:
-            print("Set 2d dataset '%s'" % name)
+            print(_("Set 2d dataset '%s'") % name)
 
     def SetDataText(self, name, val):
         """Create a text dataset."""
@@ -550,8 +612,11 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print("Set text dataset '%s'" % name)
-            print(" Values = %s" % cstr(data.data))
+            print(
+                _("Set text dataset '%s'\n"
+                  "Values = %s") % (
+                      name, repr(data.data))
+            )
 
     def GetData(self, name):
         """Return the data with the name.
@@ -608,261 +673,6 @@ class CommandInterface(qt4.QObject):
         else:
             return '1d'
 
-    def ImportString(self, descriptor, dstring, useblocks=False):
-        """Read data from the string using a descriptor.
-
-        If useblocks is set, then blank lines or the word 'no' are used
-        to split the data into blocks. Dataset names are appended with an
-        underscore and the block number (starting from 1).
-
-        Returned is a tuple (datasets, errors)
-         where datasets is a list of datasets read
-         errors is a dict of the datasets with the number of errors while
-         converting the data
-        """
-
-        params = importparams.ImportParamsSimple(
-            descriptor=descriptor,
-            datastr=dstring,
-            useblocks=useblocks)
-        op = operations.OperationDataImport(params)
-        self.document.applyOperation(op)
-
-        if self.verbose:
-            print("Imported datasets %s" % (' '.join(op.outdatasets),))
-            for name, num in citems(op.outinvalids):
-                print("%i errors encountered reading dataset %s" % (num, name))
-
-        return (op.outdatasets, op.outinvalids)
-
-    def ImportString2D(self, datasetnames, dstring, xrange=None, yrange=None,
-                       invertrows=None, invertcols=None, transpose=None):
-        """Read two dimensional data from the string specified.
-        datasetnames is a list of datasets to read from the string or a single
-        dataset name
-
-
-        xrange is a tuple containing the range of data in x coordinates
-        yrange is a tuple containing the range of data in y coordinates
-        if invertrows=True, then rows are inverted when read
-        if invertcols=True, then cols are inverted when read
-        if transpose=True, then rows and columns are swapped
-
-        """
-        
-        if isinstance(datasetnames, cbasestr):
-            datasetnames = [datasetnames]
-
-        params = importparams.ImportParams2D(
-            datasetnames=datasetnames,
-            datastr=dstring, xrange=xrange,
-            yrange=yrange, invertrows=invertrows,
-            invertcols=invertcols, transpose=transpose)
-        op = operations.OperationDataImport2D(params)
-        self.document.applyOperation(op)
-        if self.verbose:
-            print("Imported datasets %s" % (', '.join(datasetnames)))
-
-    def ImportFile2D(self, filename, datasetnames, xrange=None, yrange=None,
-                     invertrows=None, invertcols=None, transpose=None,
-                     prefix="", suffix="", encoding='utf_8',
-                     linked=False):
-        """Import two-dimensional data from a file.
-        filename is the name of the file to read
-        datasetnames is a list of datasets to read from the file, or a single
-        dataset name
-
-        xrange is a tuple containing the range of data in x coordinates
-        yrange is a tuple containing the range of data in y coordinates
-        if invertrows=True, then rows are inverted when read
-        if invertcols=True, then cols are inverted when read
-        if transpose=True, then rows and columns are swapped
-
-        prefix and suffix are prepended and appended to dataset names
-
-        encoding is encoding character set
-
-        if linked=True then the dataset is linked to the file
-        """
-
-        # look up filename on path
-        realfilename = self.findFileOnImportPath(filename)
-
-        if isinstance(datasetnames, cbasestr):
-            datasetnames = [datasetnames]
-
-        params = importparams.ImportParams2D(
-            datasetnames=datasetnames, 
-            filename=realfilename, xrange=xrange,
-            yrange=yrange, invertrows=invertrows,
-            invertcols=invertcols, transpose=transpose,
-            prefix=prefix, suffix=suffix,
-            linked=linked)
-        op = operations.OperationDataImport2D(params)
-        self.document.applyOperation(op)
-        if self.verbose:
-            print("Imported datasets %s" % (', '.join(datasetnames)))
-
-    def ImportFile(self, filename, descriptor, useblocks=False, linked=False,
-                   prefix='', suffix='', ignoretext=False, encoding='utf_8'):
-        """Read data from file with filename using descriptor.
-        If linked is True, the data won't be saved in a saved document,
-        the data will be reread from the file.
-
-        If useblocks is set, then blank lines or the word 'no' are used
-        to split the data into blocks. Dataset names are appended with an
-        underscore and the block number (starting from 1).
-
-        If prefix is set, prefix is prepended to each dataset name
-        Suffix is added to each dataset name
-        ignoretext ignores lines of text in the file
-
-        Returned is a tuple (datasets, errors)
-         where datasets is a list of datasets read
-         errors is a dict of the datasets with the number of errors while
-         converting the data
-        """
-
-        realfilename = self.findFileOnImportPath(filename)
-
-        params = importparams.ImportParamsSimple(
-            descriptor=descriptor, filename=realfilename,
-            useblocks=useblocks, linked=linked,
-            prefix=prefix, suffix=suffix,
-            ignoretext=ignoretext)
-        op = operations.OperationDataImport(params)
-        self.document.applyOperation(op)
-
-        if self.verbose:
-            print("Imported datasets %s" % (' '.join(op.outdatasets),))
-            for name, num in citems(op.outinvalids):
-                print("%i errors encountered reading dataset %s" % (num, name))
-
-        return (op.outdatasets, op.outinvalids)
-
-    def ImportFileCSV(self, filename,
-                      readrows=False,
-                      delimiter=',', textdelimiter='"',
-                      encoding='utf_8',
-                      headerignore=0, rowsignore=0,
-                      blanksaredata=False,
-                      numericlocale='en_US',
-                      dateformat='YYYY-MM-DD|T|hh:mm:ss',
-                      headermode='multi',
-                      dsprefix='', dssuffix='', prefix=None,
-                      linked=False):
-        """Read data from a comma separated file (CSV).
-
-        Data are read from filename
-        
-        readrows: if true, data are read across rather than down
-        delimiter: character for delimiting data (usually ',')
-        textdelimiter: character surrounding text (usually '"')
-        encoding: encoding used in file
-        headerignore: number of lines to ignore after header text
-        rowsignore: number of rows to ignore at top of file
-        blanksaredata: treats blank lines in csv files as blank data values
-        numericlocale: format to use for reading numbers
-        dateformat: format for interpreting dates
-        headermode: 'multi': multiple headers allowed in file
-                    '1st': first text found are headers
-                    'none': no headers, guess data and use default names
-
-        Dataset names are prepended and appended, by dsprefix and dssuffix,
-        respectively
-         (prefix is backware compatibility only, it adds an underscore
-          relative to dsprefix)
-
-        If linked is True the data are linked with the file."""
-
-        # backward compatibility
-        if prefix:
-            dsprefix = prefix + '_'
-
-        # lookup filename
-        realfilename = self.findFileOnImportPath(filename)
-
-        params = importparams.ImportParamsCSV(
-            filename=realfilename, readrows=readrows,
-            delimiter=delimiter, textdelimiter=textdelimiter,
-            encoding=encoding,
-            headerignore=headerignore, rowsignore=rowsignore,
-            blanksaredata=blanksaredata,
-            numericlocale=numericlocale, dateformat=dateformat,
-            headermode=headermode,
-            prefix=dsprefix, suffix=dssuffix,
-            linked=linked,
-            )
-        op = operations.OperationDataImportCSV(params)
-        self.document.applyOperation(op)
-
-        if self.verbose:
-            print("Imported datasets %s" % (' '.join(op.outdatasets),))
-
-        return op.outdatasets
-
-    def ImportFITSFile(self, dsname, filename, hdu,
-                       datacol = None, symerrcol = None,
-                       poserrcol = None, negerrcol = None,
-                       wcsmode = None,
-                       linked = False):
-        """Import data from a FITS file
-
-        dsname is the name of the dataset
-        filename is name of the fits file to open
-        hdu is the number/name of the hdu to access
-
-        if the hdu is a table, datacol, symerrcol, poserrcol and negerrcol
-        specify the columns containing the data, symmetric error,
-        positive and negative errors.
-
-        wcsmode is one of ('pixel', 'pixel_wcs' or 'linear_wcs'). None
-        gives 'linear_wcs'. 'pixel' mode just gives pixel values from
-        0 to maximum. 'pixel_wcs' is the pixel number relative to the
-        wcs reference pixel. 'linear_wcs' takes the wcs coordinate,
-        assuming a linear coordinate system. 'fraction' assumes
-        fractional values from 0 to 1.
-
-        linked specfies that the dataset is linked to the file
-        """
-
-        # lookup filename
-        realfilename = self.findFileOnImportPath(filename)
-        params = importparams.ImportParamsFITS(
-            dsname=dsname, filename=realfilename, hdu=hdu,
-            datacol=datacol, symerrcol=symerrcol,
-            poserrcol=poserrcol, negerrcol=negerrcol,
-            wcsmode=wcsmode,
-            linked=linked)
-        op = operations.OperationDataImportFITS(params)
-        self.document.applyOperation(op)
-
-    def ImportFilePlugin(self, plugin, filename, **args):
-        """Import file using a plugin.
-
-        optional arguments:
-        prefix: add to start of dataset name (default '')
-        suffix: add to end of dataset name (default '')
-        linked: link import to file (default False)
-        encoding: file encoding (may not be used, default 'utf_8')
-        plus arguments to plugin
-
-        returns: list of imported datasets, list of imported customs
-        """
-
-        realfilename = self.findFileOnImportPath(filename)
-        params = importparams.ImportParamsPlugin(
-            plugin=plugin, filename=realfilename, **args)
-
-        op = operations.OperationDataImportPlugin(params)
-        try:
-            self.document.applyOperation(op)
-        except:
-            self.document.log("Error in plugin %s" % plugin)
-            exc =  ''.join(traceback.format_exc())
-            self.document.log(exc)
-        return op.outdatasets, op.outcustoms
-
     def ReloadData(self):
         """Reload any linked datasets.
 
@@ -892,7 +702,7 @@ class CommandInterface(qt4.QObject):
         """Export plot to filename.
 
         color is True or False if color is requested in output file
-        page is the pagenumber to export
+        page is the pagenumber to export (or list for multipage formats)
         dpi is the number of dots per inch for bitmap output files
         antialias antialiases output if True
         quality is a quality parameter for jpeg output
@@ -901,7 +711,7 @@ class CommandInterface(qt4.QObject):
         pdfdpi is the dpi to use when exporting eps or pdf files
         svgtextastext: write text in SVG as text, rather than curves
         """
-        
+
         e = export.Export(self.document, filename, page, color=color,
                           bitmapdpi=dpi, antialias=antialias,
                           quality=quality, backcolor=backcolor,
@@ -983,5 +793,29 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print("Applied tag %s to datasets %s" % (
+            print(_("Applied tag %s to datasets %s") % (
                 tag, ' '.join(datasets)))
+
+    def FilterDatasets(self, filterexpr, datasets,
+                       prefix="", suffix="",
+                       invert=False, replaceblanks=False):
+        """Apply filter expression to list of datasets.
+
+        filterexpr: input filter expression
+        datasets: list of input dataset names
+        prefix, suffix: output prefix/suffix to add to names (one must be set)
+        invert: invert results of filter expression
+        replaceblanks: replace filtered values with nan/blank in output.
+        """
+        op = dataset_filtered.OperationDatasetsFilter(
+            filterexpr, datasets,
+            prefix=prefix, suffix=suffix,
+            invert=invert, replaceblanks=replaceblanks)
+        self.document.applyOperation(op)
+
+        if self.verbose:
+            print(
+                _('Filtered datasets %s using expression %s. '
+                  'Output prefix=%s, suffix=%s') % (
+                      datasets, filterexpr, prefix, suffix)
+            )
