@@ -127,7 +127,11 @@ class DatasetException(Exception):
     pass
 
 class DatasetBase(object):
-    """A base dataset class."""
+    """Base class for all datasets."""
+
+class DatasetConcreteBase(DatasetBase):
+    """A base dataset class for datasets which are real, and not proxies,
+    etc."""
 
     # number of dimensions the dataset holds
     dimensions = 0
@@ -198,13 +202,6 @@ class DatasetBase(object):
         group is the group to save it in (h5py group)
         """
 
-    def name(self):
-        """Get dataset name."""
-        for name, ds in citems(self.document.data):
-            if ds == self:
-                return name
-        raise ValueError('Could not find self in document.data')
-
     def userSize(self):
         """Return dimensions of dataset for user."""
         return ""
@@ -214,8 +211,8 @@ class DatasetBase(object):
         1, 2, 3, ..., 4, 5, 6."""
         return None
 
-    def description(self, showlinked=True):
-        """Get description of database."""
+    def description(self):
+        """Get description of dataset."""
         return ""
 
     def uiConvertToDataItem(self, val):
@@ -246,7 +243,7 @@ class DatasetBase(object):
 
         e.g. dataset[5:100] - make a dataset based on items 5 to 99 inclusive
         """
-        return type(self)(**self._getItemHelper(key))
+        return self.returnCopyWithNewData(**self._getItemHelper(key))
 
     def __len__(self):
         """Return length of dataset."""
@@ -279,6 +276,10 @@ class DatasetBase(object):
         """Return an unlinked copy of self."""
         pass
 
+    def returnCopyWithNewData(self, **args):
+        """Return copy with new data given."""
+        pass
+
     def renameable(self):
         """Is it possible to rename this dataset?"""
         return self.linked is None
@@ -295,7 +296,7 @@ def regularGrid(vals):
     deltas = vals[1:] - vals[:-1]
     return N.all(N.abs(deltas - deltas[0]) < (deltas[0]*1e-5))
 
-class Dataset2DBase(DatasetBase):
+class Dataset2DBase(DatasetConcreteBase):
     """Ancestor for 2D datasets."""
 
     # number of dimensions the dataset holds
@@ -415,16 +416,13 @@ class Dataset2DBase(DatasetBase):
         """Return preview of data."""
         return dsPreviewHelper(self.data.flatten())
 
-    def description(self, showlinked=True):
+    def description(self):
         """Get description of dataset."""
 
-        text = self.name()
-        text += u' (%i×%i)' % self.data.shape
         xr, yr = self.getDataRanges()
-        text += ', x=%g->%g' % tuple(xr)
-        text += ', y=%g->%g' % tuple(yr)
-        if self.linked and showlinked:
-            text += ', linked to %s' % self.linked.filename
+        text = _(u"2D (%i×%i), numeric, x=%.4g->%.4g, y=%.4g->%.4g") % (
+            self.data.shape[0], self.data.shape[1],
+            xr[0], xr[1], yr[0], yr[1])
         return text
 
     def returnCopy(self):
@@ -433,12 +431,15 @@ class Dataset2DBase(DatasetBase):
                           xedge=self.xedge, yedge=self.yedge,
                           xcent=self.xcent, ycent=self.ycent )
 
+    def returnCopyWithNewData(self, **args):
+        return Dataset2D(**args)
+
 class Dataset2D(Dataset2DBase):
     '''Represents a two-dimensional dataset.'''
 
     editable = True
 
-    def __init__(self, data, xrange=None, yrange=None,
+    def __init__(self, data=None, xrange=None, yrange=None,
                  xedge=None, yedge=None,
                  xcent=None, ycent=None):
         '''Create a two dimensional dataset based on data.
@@ -564,7 +565,7 @@ def dsPreviewHelper(d):
         return line1
     return line1 + '\n' + line2
 
-class Dataset1DBase(DatasetBase):
+class Dataset1DBase(DatasetConcreteBase):
     """Base for 1D datasets."""
 
     # number of dimensions the dataset holds
@@ -584,21 +585,16 @@ class Dataset1DBase(DatasetBase):
         """Preview of data."""
         return dsPreviewHelper(self.data)
 
-    def description(self, showlinked=True):
+    def description(self):
         """Get description of dataset."""
 
-        text = self.name()
         if self.serr is not None:
-            text += ',+-'
-        if self.perr is not None:
-            text += ',+'
-        if self.nerr is not None:
-            text += ',-'
-        text += _(' (length %i)') % len(self.data)
-
-        if self.linked and showlinked:
-            text += _(' linked to %s') % self.linked.filename
-        return text
+            templ = _("1D (length %i, symmetric errors)")
+        elif self.perr is not None or self.nerr is not None:
+            templ = _("1D (length %i, asymmetric errors)")
+        else:
+            templ = _("1D (length %i)")
+        return templ % len(self.data)
 
     def invalidDataPoints(self):
         """Return a numpy bool detailing which datapoints are invalid."""
@@ -682,6 +678,10 @@ class Dataset1DBase(DatasetBase):
                        perr = _copyOrNone(self.perr),
                        nerr = _copyOrNone(self.nerr))
 
+    def returnCopyWithNewData(self, **args):
+        """Return dataset of same type using the column data given."""
+        return Dataset(**args)
+
 class Dataset(Dataset1DBase):
     '''Represents a dataset.'''
 
@@ -796,11 +796,16 @@ class DatasetDateTimeBase(Dataset1DBase):
     dstype = _('Date')
     displaytype = 'date'
 
-    def description(self, showlinked=True):
-        text = _('%s (%i date/times)') % (self.name(), len(self.data))
-        if self.linked and showlinked:
-            text += ', linked to %s' % self.linked.filename
-        return text
+    def description(self):
+        return _('Date/time (length %i)') % len(self.data)
+
+    def returnCopy(self):
+        """Returns version of dataset with no linking."""
+        return DatasetDateTime(data=N.array(self.data))
+
+    def returnCopyWithNewData(self, **args):
+        """Return dataset of same type using the column data given."""
+        return DatasetDateTime(**args)
 
     def uiConvertToDataItem(self, val):
         """Return a value cast to this dataset data type."""
@@ -853,10 +858,6 @@ class DatasetDateTime(DatasetDateTimeBase):
         data.attrs['vsz_convert_datetime'] = 1
         data.attrs['vsz_name'] = name.encode('utf-8')
 
-    def returnCopy(self):
-        """Returns version of dataset with no linking."""
-        return DatasetDateTime(data=N.array(self.data))
-
     def deleteRows(self, row, numrows):
         """Delete numrows rows starting from row.
         Returns deleted rows as a dict of {column:data, ...}
@@ -891,7 +892,7 @@ class DatasetDateTime(DatasetDateTimeBase):
         # tell the document that we've changed
         self.document.modifiedData(self)
 
-class DatasetText(DatasetBase):
+class DatasetText(DatasetConcreteBase):
     """Represents a text dataset: holding an array of strings."""
 
     dimensions = 1
@@ -904,14 +905,11 @@ class DatasetText(DatasetBase):
     def __init__(self, data=None, linked=None):
         """Initialise dataset with data given. Data are a list of strings."""
 
-        DatasetBase.__init__(self, linked=linked)
+        DatasetConcreteBase.__init__(self, linked=linked)
         self.data = list(data)
 
-    def description(self, showlinked=True):
-        text = _('%s (%i items)') % (self.name(), len(self.data))
-        if self.linked and showlinked:
-            text += _(', linked to %s') % self.linked.filename
-        return text
+    def description(self):
+        return _('Text (length %i)') % len(self.data)
 
     def userSize(self):
         """Size of dataset."""
@@ -982,6 +980,10 @@ class DatasetText(DatasetBase):
         """Returns version of dataset with no linking."""
         return DatasetText(self.data)
 
+    def returnCopyWithNewData(self, **args):
+        """Return dataset of same type using the column data given."""
+        return DatasetText(**args)
+
 class DatasetExpressionException(DatasetException):
     """Raised if there is an error evaluating a dataset expression."""
     pass
@@ -1330,7 +1332,7 @@ class DatasetExpression(Dataset1DBase):
     def __getitem__(self, key):
         """Return a dataset based on this dataset
 
-        We override this from DatasetBase as it would return a
+        We override this from DatasetConcreteBase as it would return a
         DatsetExpression otherwise, not chopped sets of data.
         """
         return Dataset(**self._getItemHelper(key))
@@ -1391,7 +1393,7 @@ class DatasetRange(Dataset1DBase):
     def __getitem__(self, key):
         """Return a dataset based on this dataset
 
-        We override this from DatasetBase as it would return a
+        We override this from DatasetConcreteBase as it would return a
         DatsetExpression otherwise, not chopped sets of data.
         """
         return Dataset(**self._getItemHelper(key))
@@ -1468,6 +1470,10 @@ def getSpacing(data):
                     raise DatasetExpressionException(
                         'Variable spacings not yet supported '
                         'in constructing 2D datasets')
+
+    if mindelta is None or mindelta == 0:
+        raise DatasetExpressionException('Could not identify delta')
+
     return (uniquesorted[0], uniquesorted[-1], mindelta,
             int((uniquesorted[-1]-uniquesorted[0])/mindelta)+1)
 
@@ -1579,13 +1585,6 @@ class Dataset2DXYZExpression(Dataset2DBase):
             return N.array( [[]] )
         return ds
 
-    def description(self, showlinked=True):
-        # FIXME: dataeditdialog descriptions should be taken from here somewhere
-        text = self.name()
-        text += ' (%ix%i)' % self.data.shape
-        text += ', x=%g->%g' % tuple(self.xrange)
-        text += ', y=%g->%g' % tuple(self.yrange)
-
     def saveDataRelationToText(self, fileobj, name):
         '''Save expressions to file.
         '''
@@ -1841,7 +1840,7 @@ class Dataset1DPlugin(_DatasetPlugin, Dataset1DBase):
     def __getitem__(self, key):
         """Return a dataset based on this dataset
 
-        We override this from DatasetBase as it would return a
+        We override this from DatasetConcreteBase as it would return a
         DatsetExpression otherwise, not chopped sets of data.
         """
         return Dataset(**self._getItemHelper(key))
