@@ -19,8 +19,11 @@
 """A home-brewed SVG paint engine for doing svg with clipping
 and exporting text as paths for WYSIWYG."""
 
+from __future__ import division, print_function
 import re
-import veusz.qtall as qt4
+
+from ..compat import crange, cbytes
+from .. import qtall as qt4
 
 # dpi runs at many times usual, and results are scaled down
 # helps fix point issues in font sizes
@@ -32,15 +35,18 @@ inch_pt = 72.0
 
 def printpath(path):
     """Debugging print path."""
-    print "Contents of", path
-    for i in xrange(path.elementCount()):
+    print("Contents of", path)
+    for i in crange(path.elementCount()):
         el = path.elementAt(i)
-        print " ", el.type, el.x, el.y
+        print(" ", el.type, el.x, el.y)
 
 def fltStr(v, prec=2):
     """Change a float to a string, using a maximum number of decimal places
     but removing trailing zeros."""
 
+    # ensures consistent rounding behaviour on different platforms
+    v = round(v, prec+2)
+
     val = ('% 20.10f' % v)[:10+prec]
 
     # drop any trailing zeros
@@ -164,7 +170,7 @@ class SVGPaintEngine(qt4.QPaintEngine):
         self.clippath = None
         self.clipnum = 0
         self.existingclips = {}
-        self.matrix = qt4.QMatrix()
+        self.transform = qt4.QTransform()
 
         # svg root element for qt defaults
         self.rootelement = SVGElement(
@@ -238,7 +244,7 @@ class SVGPaintEngine(qt4.QPaintEngine):
     def _updateClipPath(self, clippath, clipoperation):
         """Update clip path given state change."""
 
-        clippath = self.matrix.map(clippath)
+        clippath = self.transform.map(clippath)
 
         if clipoperation == qt4.Qt.NoClip:
             self.clippath = None
@@ -258,7 +264,7 @@ class SVGPaintEngine(qt4.QPaintEngine):
         # state is a list of transform, stroke/fill and clip states
         statevec = list(self.oldstate)
         if ss & qt4.QPaintEngine.DirtyTransform:
-            self.matrix = state.matrix()
+            self.transform = state.transform()
             statevec[0] = self.transformState()
         if ss & qt4.QPaintEngine.DirtyPen:
             self.pen = state.pen()
@@ -277,18 +283,18 @@ class SVGPaintEngine(qt4.QPaintEngine):
 
         # work out which state differs first
         pop = 0
-        for i in xrange(2, -1, -1):
+        for i in crange(2, -1, -1):
             if statevec[i] != self.oldstate[i]:
                 pop = i+1
                 break
 
         # go back up the tree the required number of times
-        for i in xrange(pop):
+        for i in crange(pop):
             if self.oldstate[i]:
                 self.celement = self.celement.parent
 
         # create new elements for changed states
-        for i in xrange(pop-1, -1, -1):
+        for i in crange(pop-1, -1, -1):
             if statevec[i]:
                 self.celement = SVGElement(
                     self.celement, 'g', ' '.join(statevec[i]))
@@ -370,8 +376,8 @@ class SVGPaintEngine(qt4.QPaintEngine):
         return tuple(items)
 
     def transformState(self):
-        if not self.matrix.isIdentity():
-            m = self.matrix
+        if not self.transform.isIdentity():
+            m = self.transform
             dx, dy = m.dx(), m.dy()
             if (m.m11(), m.m12(), m.m21(), m.m22()) == (1., 0., 0., 1):
                 out = ('transform="translate(%s,%s)"' % (
@@ -446,7 +452,7 @@ class SVGPaintEngine(qt4.QPaintEngine):
                 self.celement, 'g',
                 ' '.join(grpattrb) )
 
-            text = escapeXML( unicode(textitem.text()) )
+            text = escapeXML( textitem.text() )
 
             textattrb = [
                 'x="%s"' % fltStr(pt.x()*scale),
@@ -454,11 +460,15 @@ class SVGPaintEngine(qt4.QPaintEngine):
                 'textLength="%s"' % fltStr(textitem.width()*scale),
                 ]
 
+            # spaces get lost without this
+            if text.find('  ') >= 0 or text[:1] == ' ' or text[-1:] == ' ':
+                textattrb.append('xml:space="preserve"')
+
             # write as an SVG text element
             SVGElement(
                 grp, 'text',
                 ' '.join(textattrb),
-                text=text.encode('utf-8') )
+                text=text )
 
         else:
             # convert to a path
@@ -536,7 +546,7 @@ class SVGPaintEngine(qt4.QPaintEngine):
                   'width="%s" ' % fltStr(r.width()*scale),
                   'height="%s" ' % fltStr(r.height()*scale),
                   'xlink:href="data:image/%s;base64,' % self.imageformat,
-                  str(data.toBase64()),
+                  cbytes(data.toBase64()).decode('ascii'),
                   '" preserveAspectRatio="none"' ]
         SVGElement(self.celement, 'image', ''.join(attrb))
 
