@@ -18,18 +18,20 @@
 
 """Import plugin base class and helpers."""
 
+from __future__ import division
 import os.path
 import numpy as N
-import veusz.utils as utils
-import veusz.qtall as qt4
 
-import field
-import datasetplugin
+from ..compat import crange, cstr, cstrerror
+from .. import utils
+from .. import qtall as qt4
+
+from . import field
+from . import datasetplugin
 
 def _(text, disambiguation=None, context='ImportPlugin'):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # add an instance of your class to this list to get it registered
 importpluginregistry = []
@@ -121,7 +123,7 @@ class ImportPluginExample(ImportPlugin):
             sub = float(params.field_results["subtract"])
             if params.field_results["invert"]:
                 mult *= -1
-            for i in xrange(params.field_results["skip"]):
+            for i in crange(params.field_results["skip"]):
                 f.readline()
             for line in f:
                 data += [float(x)*mult-sub for x in line.split()]
@@ -129,8 +131,8 @@ class ImportPluginExample(ImportPlugin):
             return [datasetplugin.Dataset1D(params.field_results["name"], data),
                     datasetplugin.Constant("testconst", "42"),
                     datasetplugin.Function("testfunc(x)", "testconst*x**2")]
-        except Exception, e:
-            raise ImportPluginException(unicode(e))
+        except Exception as e:
+            raise ImportPluginException(cstr(e))
 
 class ImportPluginDateTime(ImportPlugin):
     """An example plugin for reading a set of iso date-times from a
@@ -238,7 +240,7 @@ class QdpFile(object):
 
         for num, r1, c1, r2, c2 in self.datagroup2d:
             arr = []
-            for c in xrange(c1-1,c2-1+1):
+            for c in crange(c1-1,c2-1+1):
                 arr.append( self.data[c][r1-1:r2-1+1] )
                 # make data as "used"
                 self.data[c] = None
@@ -266,7 +268,7 @@ class QdpFile(object):
         """Add data to output array.
         """
 
-        for i in xrange(len(self.data)):
+        for i in crange(len(self.data)):
             if self.data[i] is None:
                 continue
 
@@ -518,9 +520,9 @@ class ImportPluginNpy(ImportPlugin):
 
         try:
             retn = N.load(params.filename)
-        except Exception, e:
+        except Exception as e:
             raise ImportPluginException(_("Error while reading file: %s") %
-                                        unicode(e))
+                                        cstr(e))
 
         return [ cnvtImportNumpyArray(
                 name, retn, errorsin2d=params.field_results["errorsin2d"]) ]
@@ -572,9 +574,9 @@ class ImportPluginNpz(ImportPlugin):
 
         try:
             retn = N.load(params.filename)
-        except Exception, e:
+        except Exception as e:
             raise ImportPluginException(_("Error while reading file: %s") %
-                                        unicode(e))
+                                        cstr(e))
 
         try:
             retn.files
@@ -623,8 +625,8 @@ class ImportPluginBinary(ImportPlugin):
             f = open(params.filename, "rb")
             data = f.read()
             f.close()
-        except EnvironmentError, e:
-            return _("Cannot read file (%s)") % utils.decodeDefault(e.strerror), False
+        except EnvironmentError as e:
+            return _("Cannot read file (%s)") % cstrerror(e), False
 
         text = [_('File length: %i bytes') % len(data)]
 
@@ -636,7 +638,7 @@ class ImportPluginBinary(ImportPlugin):
                 return c
 
         # do a hex dump (like in CP/M)
-        for i in xrange(0, min(65536, len(data)), 16):
+        for i in crange(0, min(65536, len(data)), 16):
             hdr = '%04X  ' % i
             subset = data[i:i+16]
             hexdata = ('%02X '*len(subset)) % tuple([ord(x) for x in subset])
@@ -658,24 +660,88 @@ class ImportPluginBinary(ImportPlugin):
             f.seek( params.field_results["offset"] )
             retn = f.read()
             f.close()
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             raise ImportPluginException(_("Error while reading file '%s'\n\n%s") %
-                                        (params.filename, utils.decodeDefault(e.strerror)))
+                                        (params.filename, cstrerror(e)))
 
         try:
             data = N.fromstring(retn, dtype=self.getNumpyDataType(params),
                                 count=params.field_results["length"])
-        except ValueError, e:
+        except ValueError as e:
             raise ImportPluginException(_("Error converting data for file '%s'\n\n%s") %
-                                        (params.filename, unicode(e)))
+                                        (params.filename, cstr(e)))
 
         data = data.astype(N.float64)
         return [ datasetplugin.Dataset1D(name, data) ]
 
+class ImportPluginGnuplot2D(ImportPlugin):
+    """A Veusz plugin for reading data in Gnuplot 2D data format from a file."""
+
+    name = "Gnuplot 2D data import plugin"
+    author = "Joerg Meyer, j.meyer@chem.leidenuniv.nl"
+    description = "Reads data in Gnuplot 2D format from a text file."
+
+    file_extensions = set(['.data','.elbow'])
+
+    def __init__(self):
+        ImportPlugin.__init__(self)
+        self.fields = [
+            field.FieldText(
+                "name", descr="Dataset name", default="name"),
+            field.FieldFloat(
+                "subtract", descr="Offset to subtract", default=0.0),
+            field.FieldFloat(
+                "mult", descr="Multiplication factor", default=1),
+            ]
+
+    def doImport(self, params):
+        """Actually import data
+        params is a ImportPluginParams object.
+        Return a list of ImportDataset1D, ImportDataset2D objects
+        """
+
+        sub = float(params.field_results["subtract"])
+        mult = params.field_results["mult"]
+        f = params.openFileWithEncoding()
+        data_gp = []
+        data_gp_block = []
+        for line in f:
+            fields = line.split()
+            if not fields:
+                if data_gp_block:
+                    data_gp.append( data_gp_block )
+                    data_gp_block = []
+                else:                        # ignore initial blank lines
+                    continue
+            elif '#' in fields[0]:            # ignore comment lines
+                continue
+            else:
+                x,y,z = map(float, fields[0:3])
+                data_gp_block.append( [x,y,(z-sub)*mult] )
+
+        if data_gp_block:                    # append last block if necessary
+            data_gp.append( data_gp_block )
+            data_gp_block = []
+
+        data = N.array(data_gp)
+        S = data.shape
+        data_for_sorting = data.reshape((S[0]*S[1],S[2]))
+        ind = N.lexsort( [data_for_sorting[:,0], data_for_sorting[:,1]] )
+        data_veusz = data_for_sorting[ind].reshape(S)[:,:,2]
+        rangex = (data[:,:,0].min(),data[:,:,0].max())
+        rangey = (data[:,:,1].min(),data[:,:,1].max())
+
+        return [
+            datasetplugin.Dataset2D(
+                params.field_results["name"],
+                data=data_veusz, rangex=rangex, rangey=rangey)
+        ]
+
 importpluginregistry += [
     ImportPluginNpy,
     ImportPluginNpz,
     ImportPluginQdp,
     ImportPluginBinary,
     ImportPluginExample,
+    ImportPluginGnuplot2D,
     ]
