@@ -18,18 +18,19 @@
 
 """Plugins for general operations."""
 
+from __future__ import division
 import random
 import re
 import fnmatch
 
-import veusz.qtall as qt4
-import veusz.setting as setting
-import field
+from ..compat import cbasestr
+from .. import qtall as qt4
+from .. import utils
+from . import field
 
 def _(text, disambiguation=None, context='ToolsPlugin'):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # add an instance of your class to this list to be registered
 toolspluginregistry = []
@@ -166,7 +167,7 @@ class ColorsSequence(ToolsPlugin):
 
         def colatidx(i):
             """Get color in range 0...numwidgets-1."""
-            div = float(max(numwidgets-1, 1))
+            div = max(numwidgets-1, 1)
 
             H = i * (H2-H1) / div + H1
             S = i * (S2-S1) / div + S1
@@ -320,14 +321,14 @@ class TextReplace(ToolsPlugin):
 
                 val = node.val
                 # try to change if a string, and not only strings or type is string
-                if isinstance(val, basestring) and (not fields['onlystr'] or
-                                                    node.settingtype == 'str'):
+                if isinstance(val, cbasestr) and (not fields['onlystr'] or
+                                                  node.settingtype == 'str'):
                     # update text if it changes
                     val2 = val.replace(fields['text1'], fields['text2'])
                     if val != val2:
                         try:
                             node.val = val2
-                        except setting.InvalidType:
+                        except utils.InvalidType:
                             pass
             else:
                 for c in node.children:
@@ -403,7 +404,8 @@ class WidgetsClone(ToolsPlugin):
             """Walk nodes, changing datasets."""
             if node.type == 'setting':
                 if node.settingtype in (
-                    'dataset', 'dataset-or-floatlist', 'dataset-or-str'):
+                        'dataset', 'dataset-extended',
+                        'dataset-or-floatlist', 'dataset-or-str'):
                     # handle single datasets
                     if node.val == dsname:
                         node.val = dsrepl
