@@ -18,15 +18,16 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-import veusz.qtall as qt4
+from __future__ import division
+from .. import qtall as qt4
 import numpy as N
 
 try:
-    from veusz.helpers.qtloops import plotPathsToPainter
+    from ..helpers.qtloops import plotPathsToPainter
 except ImportError:
-    from slowfuncs import plotPathsToPainter
+    from .slowfuncs import plotPathsToPainter
 
-import colormap
+from . import colormap
 
 """This is the symbol plotting part of Veusz
 
@@ -303,7 +304,9 @@ def pentagonHolePath(painter, path, size):
 
 def squareRoundedPath(painter, path, size):
     """A square with rounded corners."""
-    path.addRoundRect( qt4.QRectF(-size, -size, size*2, size*2), 50, 50 )
+    path.addRoundedRect(
+        qt4.QRectF(-size, -size, size*2, size*2),
+        50, 50, qt4.Qt.RelativeSize)
 
 def dotPath(painter, path, size):
     """Draw a dot."""
@@ -362,7 +365,7 @@ def getPainterPath(painter, name, size):
     elif name == 'none':
         return qt4.QPainterPath(), True
     else:
-        raise ValueError, "Invalid marker name %s" % name
+        raise ValueError("Invalid marker name %s" % name)
 
 #######################################################################
 ## external interfaces
@@ -400,7 +403,7 @@ MarkerCodes = (
     )
 
 def plotMarkers(painter, xpos, ypos, markername, markersize, scaling=None,
-                clip=None, cmap=None, colorvals=None):
+                clip=None, cmap=None, colorvals=None, scaleline=False):
     """Funtion to plot an array of markers on a painter.
 
     painter: QPainter
@@ -411,6 +414,7 @@ def plotMarkers(painter, xpos, ypos, markername, markersize, scaling=None,
     clip: rectangle if clipping wanted
     cmap: colormap to use if colorvals is set
     colorvals: color values 0-1 of each point if used
+    scaleline: if scaling, scale border line width with scaling
     """
 
     # minor optimization
@@ -440,7 +444,8 @@ def plotMarkers(painter, xpos, ypos, markername, markersize, scaling=None,
             cmap, 'linear', color2d, 0., 1., trans)
 
     # this is the fast (C++) or slow (python) helper
-    plotPathsToPainter(painter, path, xpos, ypos, scaling, clip, colorimg)
+    plotPathsToPainter(painter, path, xpos, ypos, scaling, clip, colorimg,
+                       scaleline)
 
     painter.restore()
 
