@@ -26,10 +26,13 @@ import re
 import os.path
 import threading
 import codecs
+import io
 import csv
+import time
 from collections import defaultdict
 
-from ..compat import citems, cstr, CStringIO, cbasestr, cpy3, cbytes
+from ..compat import citems, cstr, CStringIO, cbasestr, cpy3, cbytes, crepr, \
+    crange
 from .. import qtall as qt4
 import numpy as N
 
@@ -154,12 +157,6 @@ def pixmapAsHtml(pix):
     b64 = cbytes(buf.data().toBase64()).decode('ascii')
     return '<img src="data:image/png;base64,%s">' % b64
 
-def BoundCaller(function, *params):
-    """Wrap a function with its initial arguments."""
-    def wrapped(*args):
-        function( *(params+args) )
-    return wrapped
-
 def pythonise(text):
     """Turn an expression of the form 'A b c d' into 'A(b,c,d)'.
 
@@ -250,15 +247,19 @@ class NonBlockingReaderThread(threading.Thread):
 
     This is used mainly because windows doesn't properly support
     non-blocking pipes as files.
+
+    If exiteof is True, then exit capturing when we can capture no
+    more data.
     """
 
-    def __init__(self, fileobject):
+    def __init__(self, fileobject, exiteof=True):
         """Create the thread object."""
         threading.Thread.__init__(self)
         self.fileobject = fileobject
         self.lock = threading.Lock()
         self.data = ''
         self.done = False
+        self.exiteof = exiteof
 
     def getNewData(self):
         """Get any data waiting to be read, and whether
@@ -292,14 +293,17 @@ class NonBlockingReaderThread(threading.Thread):
 
             # no more data: end of file
             if len(data) == 0:
+                if self.exiteof:
+                    self.lock.acquire()
+                    self.done = True
+                    self.lock.release()
+                    break
+                else:
+                    time.sleep(0.1)
+            else:
                 self.lock.acquire()
-                self.done = True
+                self.data += data
                 self.lock.release()
-                break
-
-            self.lock.acquire()
-            self.data += data
-            self.lock.release()
 
 # standard python encodings
 encodings = [
@@ -333,7 +337,7 @@ def openEncoding(filename, encoding, mode='r'):
         text = qt4.QApplication.clipboard().text()
         return CStringIO(text)
     else:
-        return codecs.open(filename, mode, encoding, 'ignore')
+        return io.open(filename, mode, encoding=encoding, errors='ignore')
 
 # The following two classes are adapted from the Python documentation
 # they are modified to turn off encoding errors
@@ -522,7 +526,7 @@ def round2delt(fin1, fin2):
     else:
         out1 = out2 = ''
 
-    for i in xrange(maxlog,-200,-1):
+    for i in crange(maxlog,-200,-1):
         p = 10**i
         d1, d2 = int(f1/p), int(f2/p)
         f1 -= int(d1)*p
@@ -547,3 +551,58 @@ def round2delt(fin1, fin2):
     # convert back to float for output
     fout = float(out1)
     return fout if fin1 > 0 else -fout
+
+def checkOrder(inv):
+    """Check order of inv
+    Returns: +1: ascending order
+             -1: descending order
+              0: other, or non finite."""
+    v = N.array(inv)
+    if not N.all(N.isfinite(v)):
+        return 0
+    delta = v[1:] - v[:-1]
+    if N.all(delta > 0):
+        return 1
+    if N.all(delta < 0):
+        return -1
+    return 0
+
+def checkAscending(v):
+    """Check list of values is finite and ascending."""
+    v = N.array(v)
+    if not N.all( N.isfinite(v) ):
+        return False
+    return N.all( (v[1:] - v[:-1]) > 0 )
+
+def rrepr(val):
+    """Reproducible repr.
+
+    The idea is to make a repr which won't change. We sort dict and
+    set entries."""
+
+    if isinstance(val, dict):
+        l = [ "%s: %s" % (rrepr(k), rrepr(val[k]))
+              for k in sorted(val) ]
+        return "{%s}" % ", ".join(l)
+    elif isinstance(val, set):
+        l = [rrepr(v) for v in sorted(val)]
+        return "set([%s])" % ", ".join(l)
+    elif isinstance(val, list):
+        l = [rrepr(v) for v in val]
+        return "[%s]" % ", ".join(l)
+    else:
+        return crepr(val)
+
+def escapeHDFDataName(name):
+    """Return escaped dataset name for saving in HDF5 files.
+    This is because names cannot include / characters in HDF5
+    """
+    name = name.replace('`', '`BT')
+    name = name.replace('/', '`SL')
+    return name.encode('utf-8')
+
+def unescapeHDFDataName(name):
+    """Return original name after being escaped."""
+    name = name.replace('`SL', '/')
+    name = name.replace('`BT', '`')
+    return name
