@@ -18,21 +18,21 @@
 
 """For making box plots."""
 
+from __future__ import division
 import math
-from itertools import izip
 import numpy as N
 
-import veusz.qtall as qt4
-import veusz.setting as setting
-import veusz.document as document
-import veusz.utils as utils
+from ..compat import crange, czip
+from .. import qtall as qt4
+from .. import setting
+from .. import document
+from .. import utils
 
-from plotters import GenericPlotter
+from .plotters import GenericPlotter
 
 def _(text, disambiguation=None, context='BoxPlot'):
     """Translate text."""
-    return unicode( 
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 def percentile(sortedds, perc):
     """Given a sorted dataset, get the percentile perc.
@@ -100,7 +100,7 @@ class _Stats(object):
             self.topwhisker = percentile(cleaned, 98)
             self.botwhisker = percentile(cleaned, 2)
         else:
-            raise RuntimeError, "Invalid whisker mode"
+            raise RuntimeError("Invalid whisker mode")
 
         self.outliers = cleaned[ (cleaned < self.botwhisker) |
                                  (cleaned > self.topwhisker) ]
@@ -140,7 +140,7 @@ class BoxPlot(GenericPlotter):
                               usertext=_('Direction')), 0 )
         s.add( setting.DatasetOrStr('labels', '',
                                     descr=_('Dataset or string to label bars'),
-                                    usertext=_('Labels'), datatype='text'), 0 )
+                                    usertext=_('Labels')), 0 )
         s.add( setting.DatasetExtended(
                 'posn', '',
                 descr = _('Dataset or list of values giving '
@@ -290,8 +290,9 @@ class BoxPlot(GenericPlotter):
                 values = s.get('values').getData(doc)
                 if values:
                     for v in values:
-                        axrange[0] = min(axrange[0], N.nanmin(v.data))
-                        axrange[1] = max(axrange[1], N.nanmax(v.data))
+                        if len(v.data) > 0:
+                            axrange[0] = min(axrange[0], N.nanmin(v.data))
+                            axrange[1] = max(axrange[1], N.nanmax(v.data))
             else:
                 # update from manual entries
                 drange = self.rangeManual()
@@ -404,14 +405,14 @@ class BoxPlot(GenericPlotter):
             datasets = [ s.get(x).getData(doc) for x in
                          ('whiskermin', 'whiskermax', 'boxmin',
                           'boxmax', 'mean', 'median') ]
-            if None in datasets:
+            if N.any((d is None for d in datasets)):
                 return
 
         # get axes widgets
         axes = self.parent.getAxes( (s.xAxis, s.yAxis) )
 
         # return if there are no proper axes
-        if ( None in axes or
+        if ( axes[0] is None or axes[1] is None or
              axes[0].settings.direction != 'horizontal' or
              axes[1].settings.direction != 'vertical' ):
             return
@@ -440,7 +441,7 @@ class BoxPlot(GenericPlotter):
 
         if s.calculate:
             # calculated boxes
-            for vals, plotpos in izip(values, plotposns):
+            for vals, plotpos in czip(values, plotposns):
                 stats = _Stats()
                 stats.calculate(vals.data, s.whiskermode)
                 self.plotBox(painter, axes, plotpos, widgetposn, width,
@@ -449,7 +450,7 @@ class BoxPlot(GenericPlotter):
             # manually given boxes
             vals = [d.data for d in datasets] + [plotposns]
             lens = [len(d) for d in vals]
-            for i in xrange(min(lens)):
+            for i in crange(min(lens)):
                 stats = _Stats()
                 stats.topwhisker = vals[0][i]
                 stats.botwhisker = vals[1][i]
