@@ -25,7 +25,7 @@ import sys
 
 import numpy as N
 
-from ..compat import citems, czip, ckeys, cstr
+from ..compat import czip, cstr
 from .. import document
 from .. import setting
 from .. import utils
@@ -217,7 +217,7 @@ class Fit(FunctionPlotter):
             # build up a set of X=Y values
             loc = self.document.locale
             txt = []
-            for l, v in sorted(citems(vals)):
+            for l, v in sorted(vals.items()):
                 val = utils.formatNumber(v, '%.4Vg', locale=loc)
                 txt.append( '%s = %s' % (l, val) )
             # add chi2 output
@@ -242,7 +242,7 @@ class Fit(FunctionPlotter):
             return
 
         # populate the input parameters
-        paramnames = sorted(ckeys(s.values))
+        paramnames = sorted(s.values)
         params = N.array( [s.values[p] for p in paramnames] )
 
         # FIXME: loads of error handling!!
@@ -314,9 +314,6 @@ class Fit(FunctionPlotter):
             print("Fitting %s between %s and %s" % (s.variable, s.min, s.max))
 
         # various error checks
-        if len(xvals) == 0:
-            sys.stderr.write(_('No data values. Not fitting.\n'))
-            return
         if len(xvals) != len(yvals) or len(xvals) != len(yserr):
             sys.stderr.write(_('Fit data not equal in length. Not fitting.\n'))
             return
@@ -328,6 +325,17 @@ class Fit(FunctionPlotter):
         chi2 = 1
         dof = 1
 
+        # only consider finite values
+        finite = N.isfinite(xvals) & N.isfinite(yvals) & N.isfinite(yserr)
+        xvals = xvals[finite]
+        yvals = yvals[finite]
+        yserr = yserr[finite]
+
+        # check length after excluding non-finite values
+        if len(xvals) == 0:
+            sys.stderr.write(_('No data values. Not fitting.\n'))
+            return
+
         if minuit is not None:
             vals, chi2, dof = minuitFit(evalfunc, params, paramnames, s.values,
                                         xvals, yvals, yserr)
