@@ -18,20 +18,21 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-import veusz.qtall as qt4
-import veusz.document as document
-import veusz.setting as setting
-import veusz.utils as utils
-
-import widget
-import controlgraph
-
+from __future__ import division
 import math
 
+from ..compat import crange, citems
+from .. import qtall as qt4
+from .. import document
+from .. import setting
+from .. import utils
+
+from . import widget
+from . import controlgraph
+
 def _(text, disambiguation=None, context='Key'):
     """Translate text."""
-    return unicode( 
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 #############################################################################
 # classes for controlling key position interactively
@@ -54,17 +55,19 @@ class ControlKey(object):
         self.dims = tuple(boxdims)
         self.textheight = textheight
 
-    def createGraphicsItem(self):
-        return _GraphControlKey(self)
+    def createGraphicsItem(self, parent):
+        return _GraphControlKey(parent, self)
 
 class _GraphControlKey(qt4.QGraphicsRectItem):
     """The graphical rectangle which is dragged around to reposition
     the key."""
 
-    def __init__(self, params):
-        qt4.QGraphicsRectItem.__init__(self,
-                                       params.posn[0], params.posn[1],
-                                       params.dims[0], params.dims[1])
+    def __init__(self, parent, params):
+        qt4.QGraphicsRectItem.__init__(
+            self,
+            params.posn[0], params.posn[1],
+            params.dims[0], params.dims[1],
+            parent)
         self.params = params
 
         self.setCursor(qt4.Qt.SizeAllCursor)
@@ -89,8 +92,8 @@ class _GraphControlKey(qt4.QGraphicsRectItem):
 
         # these are special places where the key is aligned
         self.highlightpoints = {}
-        for xname, xval in xposn.iteritems():
-            for yname, yval in yposn.iteritems():
+        for xname, xval in citems(xposn):
+            for yname, yval in citems(yposn):
                 self.highlightpoints[(xname, yname)] = qt4.QPointF(xval, yval)
 
         self.updatePen()
@@ -104,7 +107,7 @@ class _GraphControlKey(qt4.QGraphicsRectItem):
 
         highlight = None
         highlightrect = qt4.QRectF(rect.left()-10, rect.top()-10, 20, 20)
-        for name, point in self.highlightpoints.iteritems():
+        for name, point in citems(self.highlightpoints):
             if highlightrect.contains(point):
                 highlight = name
                 break
@@ -237,9 +240,16 @@ class Key(widget.Widget):
                             maxval = 100,
                             formatting = True) )
 
+        s.add( setting.Bool(
+            'symbolswap',
+            False,
+            descr=_('Put key symbol on right and text on left'),
+            usertext=_('Swap symbol'),
+            formatting=True) )
+
     @classmethod
-    def allowedParentTypes(self):
-        import graph
+    def allowedParentTypes(klass):
+        from . import graph
         return (graph.Graph,)
 
     @staticmethod
@@ -279,7 +289,7 @@ class Key(widget.Widget):
             return (list(), (0, 0))
         
         # start with evenly-sized rows and expand to fit
-        numrows = totallines / numcols
+        numrows = totallines // numcols
         layout = []
         
         while not layout:
@@ -294,20 +304,20 @@ class Key(widget.Widget):
         while colstats[-1] == 0:
             # shift 1 item to the right, up to the first column that has
             # excess items
-            meanoccupation = max(1, sum(colstats)/float(numcols))
+            meanoccupation = max(1, sum(colstats)/numcols)
             
             # loop until we find a victim item which can be safely moved
             victimcol = numcols
             while True:
                 # find the right-most column with excess occupation number
-                for i in reversed(xrange(victimcol)):
+                for i in reversed(crange(victimcol)):
                     if colstats[i] > meanoccupation:
                         victimcol = i
                         break
                 
                 # find the last item in the victim column
                 victim = 0
-                for i in reversed(xrange(len(layout))):
+                for i in reversed(crange(len(layout))):
                     if layout[i][2] == victimcol:
                         victim = i
                         break
@@ -375,14 +385,14 @@ class Key(widget.Widget):
                 continue
             if not c.settings.hide:
                 # add an entry for each key entry for each widget
-                for i in xrange(num):
+                for i in crange(num):
                     lines = 1
                     if showtext:
                         w, h = utils.Renderer(painter, font, 0, 0,
                                               c.getKeyText(i)).getDimensions()
                         maxwidth = max(maxwidth, w)
-                        lines = max(1, math.ceil(float(h)/float(height)))
-                    
+                        lines = max(1, math.ceil(h/height))
+
                     totallines += lines
                     entries.append( (c, i, lines) )
 
@@ -450,6 +460,8 @@ class Key(widget.Widget):
 
         textpen = s.get('Text').makeQPen()
 
+        swap = s.symbolswap
+
         # plot dataset entries
         for (plotter, num, xp, yp, lines) in layout:
             xpos = x + xp*(maxwidth+2*height+symbolwidth)
@@ -462,18 +474,28 @@ class Key(widget.Widget):
                 keyoffset = (lines-1)*height/2.0
             elif s.keyAlign == 'bottom':
                 keyoffset = (lines-1)*height
-            
-            plotter.drawKeySymbol(num, painter, xpos, ypos+keyoffset,
+
+            sx = xpos
+            if swap:
+                sx += maxwidth + height
+            plotter.drawKeySymbol(num, painter, sx, ypos+keyoffset,
                                   symbolwidth, height)
             painter.restore()
 
             # write key text
             if showtext:
                 painter.setPen(textpen)
+                if swap:
+                    lx = xpos + maxwidth
+                    alignx = 1
+                else:
+                    lx = xpos + height + symbolwidth
+                    alignx = -1
+
                 utils.Renderer(painter, font,
-                               xpos + height + symbolwidth, ypos,
+                               lx, ypos,
                                plotter.getKeyText(num),
-                               -1, 1).render()
+                               alignx, 1).render()
 
         phelper.setControlGraph(
             self, [ControlKey(self, parentposn, boxposn, boxdims, height)] )
