@@ -19,14 +19,19 @@
 
 """Implements the main window of the application."""
 
-from __future__ import division
+from __future__ import division, print_function
+import os
 import os.path
 import sys
-import traceback
 import glob
-import codecs
+import re
 
-from ..compat import citems, ckeys, cstr, cexec, cstrerror
+try:
+    import h5py
+except ImportError:
+    h5py = None
+
+from ..compat import cstr, cstrerror
 from .. import qtall as qt4
 
 from .. import document
@@ -40,21 +45,6 @@ from . import plotwindow
 from . import treeeditwindow
 from .datanavigator import DataNavigatorWindow
 
-from ..dialogs.aboutdialog import AboutDialog
-from ..dialogs.reloaddata import ReloadData
-from ..dialogs.datacreate import DataCreateDialog
-from ..dialogs.datacreate2d import DataCreate2DDialog
-from ..dialogs.preferences import PreferencesDialog
-from ..dialogs.errorloading import ErrorLoadingDialog
-from ..dialogs.capturedialog import CaptureDialog
-from ..dialogs.stylesheet import StylesheetDialog
-from ..dialogs.custom import CustomDialog
-from ..dialogs.safetyimport import SafetyImportDialog
-from ..dialogs.histodata import HistoDataDialog
-from ..dialogs.plugin import handlePlugin
-from ..dialogs import importdialog
-from ..dialogs import dataeditdialog
-
 def _(text, disambiguation=None, context='MainWindow'):
     """Translate text."""
     return qt4.QCoreApplication.translate(context, text, disambiguation)
@@ -75,7 +65,7 @@ class DBusWinInterface(vzdbus.Object):
     @vzdbus.method(dbus_interface=interface, out_signature='as')
     def GetActions(self):
         """Get list of actions which can be activated."""
-        return sorted(ckeys(self.actions))
+        return sorted(self.actions)
 
     @vzdbus.method(dbus_interface=interface, in_signature='s')
     def TriggerAction(self, action):
@@ -85,6 +75,11 @@ class DBusWinInterface(vzdbus.Object):
 class MainWindow(qt4.QMainWindow):
     """ The main window class for the application."""
 
+    # this is emitted when a dialog is opened by the main window
+    dialogShown = qt4.pyqtSignal(qt4.QWidget)
+    # emitted when a document is opened
+    documentOpened = qt4.pyqtSignal()
+
     windows = []
     @classmethod
     def CreateWindow(cls, filename=None):
@@ -174,8 +169,7 @@ class MainWindow(qt4.QMainWindow):
 
         # plot queue - how many plots are currently being drawn
         self.plotqueuecount = 0
-        self.connect( self.plot, qt4.SIGNAL("queuechange"),
-                      self.plotQueueChanged )
+        self.plot.sigQueueChange.connect(self.plotQueueChanged)
         self.plotqueuelabel = qt4.QLabel()
         self.plotqueuelabel.setToolTip(_("Number of rendering jobs remaining"))
         statusbar.addWidget(self.plotqueuelabel)
@@ -194,35 +188,26 @@ class MainWindow(qt4.QMainWindow):
 
         # working directory - use previous one
         self.dirname = setdb.get('dirname', qt4.QDir.homePath())
-        self.dirname_export = setdb.get('dirname_export', self.dirname)
         if setdb['dirname_usecwd']:
-            self.dirname = self.dirname_export = os.getcwd()
+            self.dirname = os.getcwd()
 
         # connect plot signals to main window
-        self.connect( self.plot, qt4.SIGNAL("sigUpdatePage"),
-                      self.slotUpdatePage )
-        self.connect( self.plot, qt4.SIGNAL("sigAxisValuesFromMouse"),
-                      self.slotUpdateAxisValues )
-        self.connect( self.plot, qt4.SIGNAL("sigPickerEnabled"),
-                      self.slotPickerEnabled )
-        self.connect( self.plot, qt4.SIGNAL("sigPointPicked"),
-                      self.slotUpdatePickerLabel )
+        self.plot.sigUpdatePage.connect(self.slotUpdatePage)
+        self.plot.sigAxisValuesFromMouse.connect(self.slotUpdateAxisValues)
+        self.plot.sigPickerEnabled.connect(self.slotPickerEnabled)
+        self.plot.sigPointPicked.connect(self.slotUpdatePickerLabel)
 
         # disable save if already saved
         self.document.signalModified.connect(self.slotModifiedDoc)
         # if the treeeditwindow changes the page, change the plot window
-        self.connect( self.treeedit, qt4.SIGNAL("sigPageChanged"),
-                      self.plot.setPageNumber )
+        self.treeedit.sigPageChanged.connect(self.plot.setPageNumber)
 
         # if a widget in the plot window is clicked by the user
-        self.connect( self.plot, qt4.SIGNAL("sigWidgetClicked"),
-                      self.treeedit.selectWidget )
-        self.connect( self.treeedit, qt4.SIGNAL("widgetsSelected"),
-                      self.plot.selectedWidgets )
+        self.plot.sigWidgetClicked.connect(self.treeedit.selectWidget)
+        self.treeedit.widgetsSelected.connect(self.plot.selectedWidgets)
 
         # enable/disable undo/redo
-        self.connect(self.menus['edit'], qt4.SIGNAL('aboutToShow()'),
-                     self.slotAboutToShowEdit)
+        self.menus['edit'].aboutToShow.connect(self.slotAboutToShowEdit)
 
         #Get the list of recently opened files
         self.populateRecentFiles()
@@ -230,14 +215,16 @@ class MainWindow(qt4.QMainWindow):
         self.defineViewWindowMenu()
 
         # if document requests it, ask whether an allowed import
-        self.connect(self.document, qt4.SIGNAL('check_allowed_imports'),
-                     self.slotAllowedImportsDoc)
+        self.document.sigAllowedImports.connect(self.slotAllowedImportsDoc)
 
         # add on dbus interface
         self.dbusdocinterface = document.DBusInterface(self.document)
         self.dbuswininterface = DBusWinInterface(
             self.vzactions, self.dbusdocinterface.index)
 
+        # has the document already been setup
+        self.documentsetup = False
+
     def updateStatusbar(self, text):
         '''Display text for a set period.'''
         self.statusBar().showMessage(text, 2000)
@@ -276,12 +263,16 @@ class MainWindow(qt4.QMainWindow):
     def setupDefaultDoc(self):
         """Setup default document."""
 
-        # add page and default graph
-        self.document.makeDefaultDoc()
+        if not self.documentsetup:
+            # add page and default graph
+            self.document.makeDefaultDoc()
 
-        # load defaults if set
-        self.loadDefaultStylesheet()
-        self.loadDefaultCustomDefinitions()
+            # load defaults if set
+            self.loadDefaultStylesheet()
+            self.loadDefaultCustomDefinitions()
+
+            # done setup
+            self.documentsetup = True
 
     def loadDefaultStylesheet(self):
         """Loads the default stylesheet for the new document."""
@@ -348,15 +339,18 @@ class MainWindow(qt4.QMainWindow):
             self.document.redoOperation()
 
     def slotEditPreferences(self):
+        from ..dialogs.preferences import PreferencesDialog
         dialog = PreferencesDialog(self)
         dialog.exec_()
 
     def slotEditStylesheet(self):
+        from ..dialogs.stylesheet import StylesheetDialog
         dialog = StylesheetDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotEditCustom(self):
+        from ..dialogs.custom import CustomDialog
         dialog = CustomDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
@@ -369,12 +363,14 @@ class MainWindow(qt4.QMainWindow):
         menuname: string giving prefix for new menu entries (inside actions)
         """
 
+        def getLoadDialog(pluginkls):
+            def _loadPlugin():
+                from ..dialogs.plugin import handlePlugin
+                handlePlugin(self, self.document, pluginkls)
+            return _loadPlugin
+
         menu = []
         for pluginkls in pluginlist:
-            def loaddialog(pluginkls=pluginkls):
-                """Load plugin dialog"""
-                handlePlugin(self, self.document, pluginkls)
-
             actname = menuname + '.' + '.'.join(pluginkls.menu)
             text = pluginkls.menu[-1]
             if pluginkls.has_parameters:
@@ -383,7 +379,7 @@ class MainWindow(qt4.QMainWindow):
                 self,
                 pluginkls.description_short,
                 text,
-                loaddialog)
+                getLoadDialog(pluginkls))
 
             # build up menu from tuple of names
             menulook = menu
@@ -431,7 +427,7 @@ class MainWindow(qt4.QMainWindow):
                   self.slotFilePrint,
                   icon='kde-document-print', key='Ctrl+P'),
             'file.export':
-                a(self, _('Export the current page'), _('&Export...'),
+                a(self, _('Export to graphics formats'), _('&Export...'),
                   self.slotFileExport,
                   icon='kde-document-export'),
             'file.close':
@@ -504,7 +500,7 @@ class MainWindow(qt4.QMainWindow):
                   self.slotDataImport, icon='kde-vzdata-import'),
             'data.edit':
                 a(self, _('Edit and enter new datasets'), _('&Editor...'),
-                  self.slotDataEdit, icon='kde-edit-veuszedit'),
+                  lambda: self.slotDataEdit(), icon='kde-edit-veuszedit'),
             'data.create':
                 a(self, _('Create new datasets using ranges, parametrically or as functions of existing datasets'), _('&Create...'),
                   self.slotDataCreate, icon='kde-dataset-new-veuszedit'),
@@ -514,6 +510,9 @@ class MainWindow(qt4.QMainWindow):
             'data.capture':
                 a(self, _('Capture remote data'), _('Ca&pture...'),
                   self.slotDataCapture, icon='veusz-capture-data'),
+            'data.filter':
+                a(self, _('Filter data'), _('&Filter...'),
+                  self.slotDataFilter, icon='kde-filter'),
             'data.histogram':
                 a(self, _('Histogram data'), _('&Histogram...'),
                   self.slotDataHistogram, icon='button_bar'),
@@ -553,10 +552,11 @@ class MainWindow(qt4.QMainWindow):
         tb.setIconSize(qt4.QSize(iconsize, iconsize))
         tb.setObjectName('veuszdatatoolbar')
         self.addToolBar(qt4.Qt.TopToolBarArea, tb)
-        utils.addToolbarActions(tb, self.vzactions,
-                                ('data.import', 'data.edit',
-                                 'data.create', 'data.capture',
-                                 'data.reload'))
+        utils.addToolbarActions(
+            tb, self.vzactions,
+            ('data.import', 'data.edit',
+             'data.create', 'data.capture',
+             'data.filter', 'data.reload'))
 
         # menu structure
         filemenu = [
@@ -598,7 +598,7 @@ class MainWindow(qt4.QMainWindow):
         datamenu = [
             ['data.ops', _('&Operations'), datapluginsmenu],
             'data.import', 'data.edit', 'data.create',
-            'data.create2d', 'data.capture', 'data.histogram',
+            'data.create2d', 'data.capture', 'data.filter', 'data.histogram',
             'data.reload',
             ]
         helpmenu = [
@@ -639,7 +639,12 @@ class MainWindow(qt4.QMainWindow):
     def populateExamplesMenu(self):
         """Add examples to help menu."""
 
-        examples = glob.glob(os.path.join(utils.exampleDirectory, '*.vsz'))
+        # not cstr here forces to unicode for Python 2, getting
+        # filenames in unicode
+        examples = [ os.path.join(utils.exampleDirectory, f)
+                     for f in os.listdir(cstr(utils.exampleDirectory))
+                     if os.path.splitext(f)[1] == ".vsz" ]
+
         menu = self.menus["help.examples"]
         for ex in sorted(examples):
             name = os.path.splitext(os.path.basename(ex))[0]
@@ -676,12 +681,11 @@ class MainWindow(qt4.QMainWindow):
             a = self.vzactions[act]
             fn = viewHideWindow(win)
             self.viewwinfns.append( (win, a, fn) )
-            self.connect(a, qt4.SIGNAL('triggered()'), fn)
+            a.triggered.connect(fn)
 
         # needs to update state every time menu is shown
-        self.connect(self.menus['view.viewwindows'],
-                     qt4.SIGNAL('aboutToShow()'),
-                     self.slotAboutToShowViewWindow)
+        self.menus['view.viewwindows'].aboutToShow.connect(
+            self.slotAboutToShowViewWindow)
 
     def slotAboutToShowViewWindow(self):
         """Enable/disable View->Window item check boxes."""
@@ -691,10 +695,10 @@ class MainWindow(qt4.QMainWindow):
 
     def showDialog(self, dialog):
         """Show dialog given."""
-        self.connect(dialog, qt4.SIGNAL('dialogFinished'), self.deleteDialog)
+        dialog.dialogFinished.connect(self.deleteDialog)
         self.dialogs.append(dialog)
         dialog.show()
-        self.emit( qt4.SIGNAL('dialogShown'), dialog )
+        self.dialogShown.emit(dialog)
 
     def deleteDialog(self, dialog):
         """Remove dialog from list of dialogs."""
@@ -706,6 +710,7 @@ class MainWindow(qt4.QMainWindow):
 
     def slotDataImport(self):
         """Display the import data dialog."""
+        from ..dialogs import importdialog
         dialog = importdialog.ImportDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
@@ -715,6 +720,7 @@ class MainWindow(qt4.QMainWindow):
 
         If editdataset is set to a dataset name, edit this dataset
         """
+        from ..dialogs import dataeditdialog
         dialog = dataeditdialog.DataEditDialog(self, self.document)
         self.showDialog(dialog)
         if editdataset is not None:
@@ -723,30 +729,42 @@ class MainWindow(qt4.QMainWindow):
 
     def slotDataCreate(self):
         """Create new datasets."""
+        from ..dialogs.datacreate import DataCreateDialog
         dialog = DataCreateDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotDataCreate2D(self):
         """Create new datasets."""
+        from ..dialogs.datacreate2d import DataCreate2DDialog
         dialog = DataCreate2DDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotDataCapture(self):
         """Capture remote data."""
+        from ..dialogs.capturedialog import CaptureDialog
         dialog = CaptureDialog(self.document, self)
         self.showDialog(dialog)
         return dialog
 
+    def slotDataFilter(self):
+        """Filter datasets."""
+        from ..dialogs.filterdialog import FilterDialog
+        dialog = FilterDialog(self, self.document)
+        self.showDialog(dialog)
+        return dialog
+
     def slotDataHistogram(self):
         """Histogram data."""
+        from ..dialogs.histodata import HistoDataDialog
         dialog = HistoDataDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotDataReload(self):
         """Reload linked datasets."""
+        from ..dialogs.reloaddata import ReloadData
         dialog = ReloadData(self.document, self)
         self.showDialog(dialog)
         return dialog
@@ -792,6 +810,7 @@ class MainWindow(qt4.QMainWindow):
 
     def slotHelpAbout(self):
         """Show about dialog."""
+        from ..dialogs.aboutdialog import AboutDialog
         AboutDialog(self).exec_()
 
     def queryOverwrite(self):
@@ -831,14 +850,13 @@ class MainWindow(qt4.QMainWindow):
 
         # store working directory
         setdb['dirname'] = self.dirname
-        setdb['dirname_export'] = self.dirname_export
 
         # store the current geometry in the settings database
         geometry = ( self.x(), self.y(), self.width(), self.height() )
         setdb['geometry_mainwindow'] = geometry
 
         # store docked windows
-        data = str(self.saveState())
+        data = self.saveState().data()
         setdb['geometry_mainwindowstate'] = data
 
         # save current setting db
@@ -860,7 +878,10 @@ class MainWindow(qt4.QMainWindow):
             geometry = setdb['geometry_mainwindow']
             self.resize( qt4.QSize(geometry[2], geometry[3]) )
             if nummain <= 1:
-                self.move( qt4.QPoint(geometry[0], geometry[1]) )
+                geomrect = qt4.QApplication.desktop().availableGeometry()
+                newpos = qt4.QPoint(geometry[0], geometry[1])
+                if geomrect.contains(newpos):
+                    self.move(newpos)
 
         # restore docked window geometry
         if 'geometry_mainwindowstate' in setdb:
@@ -880,8 +901,9 @@ class MainWindow(qt4.QMainWindow):
             # show busy cursor
             qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
             try:
-                ofile = open(self.filename, 'w')
-                self.document.saveToFile(ofile)
+                ext = os.path.splitext(self.filename)[1]
+                mode = 'hdf5' if ext == '.vszh5' else 'vsz'
+                self.document.save(self.filename, mode)
                 self.updateStatusbar(_("Saved to %s") % self.filename)
             except EnvironmentError as e:
                 qt4.QApplication.restoreOverrideCursor()
@@ -906,14 +928,26 @@ class MainWindow(qt4.QMainWindow):
         text = u'•' * self.plotqueuecount
         self.plotqueuelabel.setText(text)
 
-    def _fileSaveDialog(self, filetype, filedescr, dialogtitle):
-        """A generic file save dialog for exporting / saving."""
+    def fileSaveDialog(self, filters, dialogtitle):
+        """A generic file save dialog for exporting / saving.
+
+        filters: list of filters
+        """
 
         fd = qt4.QFileDialog(self, dialogtitle)
         fd.setDirectory(self.dirname)
-        fd.setFileMode( qt4.QFileDialog.AnyFile )
-        fd.setAcceptMode( qt4.QFileDialog.AcceptSave )
-        fd.setFilter( "%s (*.%s)" % (filedescr, filetype) )
+        fd.setFileMode(qt4.QFileDialog.AnyFile)
+        fd.setAcceptMode(qt4.QFileDialog.AcceptSave)
+        fd.setNameFilters(filters)
+
+        # selected filetype is saved under a key constructed here
+        filetype_re = re.compile(r'.*\(\*\.([a-z0-9]+)\)')
+        filtertypes = [filetype_re.match(f).group(1) for f in filters]
+        filterkey = '_'.join(['filterdefault'] + filtertypes)
+        if filterkey in setting.settingdb:
+            filter = setting.settingdb[filterkey]
+            if filter in filters:
+                fd.selectNameFilter(filter)
 
         # okay was selected (and is okay to overwrite if it exists)
         if fd.exec_() == qt4.QDialog.Accepted:
@@ -921,20 +955,25 @@ class MainWindow(qt4.QMainWindow):
             self.dirname = fd.directory().absolutePath()
             # update the edit box
             filename = fd.selectedFiles()[0]
-            if os.path.splitext(filename)[1] == '':
+            filetype = filetype_re.match(fd.selectedNameFilter()).group(1)
+            if os.path.splitext(filename)[1][1:] != filetype:
                 filename += '.' + filetype
-
+            setting.settingdb[filterkey] = fd.selectedNameFilter()
             return filename
+
         return None
 
-    def _fileOpenDialog(self, filetype, filedescr, dialogtitle):
-        """Display an open dialog and return a filename."""
+    def fileOpenDialog(self, filters, dialogtitle):
+        """Display an open dialog and return a filename.
+
+        filters: list of filters in format "Filetype (*.vsz)"
+        """
 
         fd = qt4.QFileDialog(self, dialogtitle)
         fd.setDirectory(self.dirname)
         fd.setFileMode( qt4.QFileDialog.ExistingFile )
         fd.setAcceptMode( qt4.QFileDialog.AcceptOpen )
-        fd.setFilter( "%s (*.%s)" % (filedescr, filetype) )
+        fd.setNameFilters(filters)
 
         # if the user chooses a file
         if fd.exec_() == qt4.QDialog.Accepted:
@@ -943,7 +982,8 @@ class MainWindow(qt4.QMainWindow):
 
             filename = fd.selectedFiles()[0]
             try:
-                open(filename)
+                with open(filename):
+                    pass
             except EnvironmentError as e:
                 qt4.QMessageBox.critical(
                     self, _("Error - Veusz"),
@@ -956,7 +996,10 @@ class MainWindow(qt4.QMainWindow):
     def slotFileSaveAs(self):
         """Save As file."""
 
-        filename = self._fileSaveDialog('vsz', _('Veusz script files'), _('Save as'))
+        filters = [_('Veusz document files (*.vsz)')]
+        if h5py is not None:
+            filters += [_('Veusz HDF5 document files (*.vszh5)')]
+        filename = self.fileSaveDialog(filters, _('Save as'))
         if filename:
             self.filename = filename
             self.updateTitlebar()
@@ -975,144 +1018,84 @@ class MainWindow(qt4.QMainWindow):
             # create a new window
             self.CreateWindow(filename)
 
-    class _unsafeCmdMsgBox(qt4.QMessageBox):
-        """Show document is unsafe."""
-        def __init__(self, window, filename):
-            qt4.QMessageBox.__init__(self, _("Unsafe code in document"),
-                                     _("The document '%s' contains potentially "
-                                       "unsafe code which may damage your "
-                                       "computer or data. Please check that the "
-                                       "file comes from a "
-                                       "trusted source.") % filename,
-                                     qt4.QMessageBox.Warning,
-                                     qt4.QMessageBox.Yes,
-                                     qt4.QMessageBox.No | qt4.QMessageBox.Default,
-                                     qt4.QMessageBox.NoButton,
-                                     window)
-            self.setButtonText(qt4.QMessageBox.Yes, _("C&ontinue anyway"))
-            self.setButtonText(qt4.QMessageBox.No, _("&Stop loading"))
-
-    class _unsafeVeuszCmdMsgBox(qt4.QMessageBox):
-        """Show document has unsafe Veusz commands."""
-        def __init__(self, window):
-            qt4.QMessageBox.__init__(self, _('Unsafe Veusz commands'),
-                                     _('This Veusz document contains potentially'
-                                       ' unsafe Veusz commands for Saving, '
-                                       'Exporting or Printing. Please check that the'
-                                       ' file comes from a trusted source.'),
-                                     qt4.QMessageBox.Warning,
-                                     qt4.QMessageBox.Yes,
-                                     qt4.QMessageBox.No | qt4.QMessageBox.Default,
-                                     qt4.QMessageBox.NoButton,
-                                     window)
-            self.setButtonText(qt4.QMessageBox.Yes, _("C&ontinue anyway"))
-            self.setButtonText(qt4.QMessageBox.No, _("&Ignore command"))
-
-    def openFileInWindow(self, filename):
-        """Actually do the work of loading a new document.
+    def loadDocument(self, filename):
+        """Load a Veusz document.
+        Return True if loaded ok
         """
 
-        # FIXME: This function suffers from spaghetti code
-        # it needs splitting up into bits to make it clearer
-        # the steps are fairly well documented below, however
-        #####################################################
+        class _unsafeCmdMsgBox(qt4.QMessageBox):
+            """Show document is unsafe."""
+            def __init__(self, window):
+                qt4.QMessageBox.__init__(
+                    self, _("Unsafe code in document"),
+                    _("The document '%s' contains potentially unsafe code "
+                      "which may damage your computer or data. Please check "
+                      "that the file comes from a trusted source.") % filename,
+                    qt4.QMessageBox.Warning,
+                    qt4.QMessageBox.Yes,
+                    qt4.QMessageBox.No | qt4.QMessageBox.Default,
+                    qt4.QMessageBox.NoButton,
+                    window)
+                self.setButtonText(qt4.QMessageBox.Yes, _("C&ontinue anyway"))
+                self.setButtonText(qt4.QMessageBox.No, _("&Stop loading"))
 
-        qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
-
-        # read script
-        try:
-            script = codecs.open(filename, 'rU', encoding='utf8').read()
-        except EnvironmentError as e:
+        def _callbackunsafe():
             qt4.QApplication.restoreOverrideCursor()
-            qt4.QMessageBox.critical(
-                self, _("Error - Veusz"),
-                _("Cannot open document '%s'\n\n%s") %
-                (filename, cstrerror(e)))
-            self.setupDefaultDoc()
-            return
-
-        def errordialog(e):
-            # display error dialog if there is an error loading
-            qt4.QApplication.restoreOverrideCursor()
-            i = sys.exc_info()
-            backtrace = traceback.format_exception( *i )
-            d = ErrorLoadingDialog(self, filename, str(e), ''.join(backtrace))
-            d.exec_()
-
-        # compile script and check for security (if reqd)
-        unsafe = setting.transient_settings['unsafe_mode']
-        while True:
-            try:
-                compiled = utils.compileChecked(script, mode='exec', filename=filename,
-                                                ignoresecurity=unsafe)
-                break
-            except utils.SafeEvalException:
-                # ask the user whether to execute in unsafe mode
-                qt4.QApplication.restoreOverrideCursor()
-                if ( self._unsafeCmdMsgBox(self, filename).exec_() ==
-                     qt4.QMessageBox.No ):
-                    return
-                unsafe = True
-            except Exception as e:
-                errordialog(e)
-                return
-
-        # set up environment to run script
-        env = self.document.eval_context.copy()
-        interface = document.CommandInterface(self.document)
-
-        # allow safe commands as-is
-        for cmd in interface.safe_commands:
-            env[cmd] = getattr(interface, cmd)
-
-        # define root node
-        env['Root'] = interface.Root
-
-        # wrap "unsafe" commands with a message box to check the user
-        safenow = [unsafe]
-        def _unsafeCaller(func):
-            def wrapped(*args, **argsk):
-                if not safenow[0]:
-                    qt4.QApplication.restoreOverrideCursor()
-                    if ( self._unsafeVeuszCmdMsgBox(self).exec_() ==
-                         qt4.QMessageBox.No ):
-                        return
-                safenow[0] = True
-                func(*args, **argsk)
-            return wrapped
-        for name in interface.unsafe_commands:
-            env[name] = _unsafeCaller(getattr(interface, name))
+            v = _unsafeCmdMsgBox(self).exec_()
+            qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
+            return v == qt4.QMessageBox.Yes
 
         # save stdout and stderr, then redirect to console
         stdout, stderr = sys.stdout, sys.stderr
         sys.stdout = self.console.con_stdout
         sys.stderr = self.console.con_stderr
 
-        # get ready to load document
-        env['__file__'] = os.path.abspath(filename)
-        self.document.wipe()
-        self.document.suspendUpdates()
-
-        # allow import to happen relative to loaded file
-        interface.AddImportPath( os.path.dirname(os.path.abspath(filename)) )
+        qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
 
         try:
-            # actually run script text
-            cexec(compiled, env)
-        except Exception as e:
-            # need to remember to restore stdout, stderr
-            sys.stdout, sys.stderr = stdout, stderr
-            self.document.enableUpdates()
-            errordialog(e)
-            return
+            # get loading mode
+            ext = os.path.splitext(filename)[1].lower()
+            if ext in ('.vsz', '.py'):
+                mode = 'vsz'
+            elif ext in ('.h5', '.hdf5', '.he5', '.vszh5'):
+                mode = 'hdf5'
+            else:
+                raise document.LoadError(
+                    _("Did not recognise file type '%s'") % ext)
+
+            # do the actual loading
+            self.document.load(
+                filename,
+                mode=mode,
+                callbackunsafe=_callbackunsafe)
+
+        except document.LoadError as e:
+            from ..dialogs.errorloading import ErrorLoadingDialog
+            qt4.QApplication.restoreOverrideCursor()
+            if e.backtrace:
+                d = ErrorLoadingDialog(self, filename, cstr(e), e.backtrace)
+                d.exec_()
+            else:
+                qt4.QMessageBox.critical(
+                    self, _("Error opening %s - Veusz") % filename,
+                    cstr(e))
+            return False
+
+        qt4.QApplication.restoreOverrideCursor()
 
         # need to remember to restore stdout, stderr
         sys.stdout, sys.stderr = stdout, stderr
 
-        # document is loaded
-        self.document.enableUpdates()
-        self.document.setModified(False)
-        self.document.clearHistory()
+        self.documentsetup = True
+        return True
+
+    def openFileInWindow(self, filename):
+        """Actually do the work of loading a new document.
+        """
+
+        ok = self.loadDocument(filename)
+        if not ok:
+            return
 
         # remember file for recent list
         self.addRecentFile(filename)
@@ -1125,11 +1108,9 @@ class MainWindow(qt4.QMainWindow):
         # use current directory of file if not using cwd mode
         if not setdb['dirname_usecwd']:
             self.dirname = os.path.dirname( os.path.abspath(filename) )
-            self.dirname_export = self.dirname
 
         # notify cmpts which need notification that doc has finished opening
-        self.emit(qt4.SIGNAL("documentopened"))
-        qt4.QApplication.restoreOverrideCursor()
+        self.documentOpened.emit()
 
     def addRecentFile(self, filename):
         """Add a file to the recent files list."""
@@ -1146,7 +1127,13 @@ class MainWindow(qt4.QMainWindow):
     def slotFileOpen(self):
         """Open an existing file in a new window."""
 
-        filename = self._fileOpenDialog('vsz', _('Veusz script files'), _('Open'))
+        filters = ['*.vsz']
+        if h5py is not None:
+            filters.append('*.vszh5')
+
+        filename = self.fileOpenDialog(
+            [_('Veusz document files (%s)') % ' '.join(filters)],
+            _('Open'))
         if filename:
             self.openFile(filename)
 
@@ -1154,131 +1141,39 @@ class MainWindow(qt4.QMainWindow):
         """Populate the recently opened files menu with a list of
         recently opened files"""
 
+        def opener(path):
+            def _fileOpener():
+                self.openFile(path)
+            return _fileOpener
+
         menu = self.menus["file.filerecent"]
         menu.clear()
 
-        newMenuItems = []
         if setdb['main_recentfiles']:
             files = [f for f in setdb['main_recentfiles']
                      if os.path.isfile(f)]
-            self._openRecentFunctions = []
 
             # add each recent file to menu
+            newmenuitems = []
             for i, path in enumerate(files):
-
-                def fileOpener(filename=path):
-                    self.openFile(filename)
-
-                self._openRecentFunctions.append(fileOpener)
-                newMenuItems.append(('filerecent%i' % i, _('Open File %s') % path,
-                                     os.path.basename(path),
-                                     'file.filerecent', fileOpener,
-                                     '', False, ''))
+                newmenuitems.append(
+                    ('filerecent%i' % i,_('Open File %s') % path,
+                     os.path.basename(path),
+                     'file.filerecent', opener(path),
+                     '', False, ''))
 
             menu.setEnabled(True)
             self.recentFileActions = utils.populateMenuToolbars(
-                newMenuItems, self.maintoolbar, self.menus)
+                newmenuitems, self.maintoolbar, self.menus)
         else:
             menu.setEnabled(False)
 
     def slotFileExport(self):
         """Export the graph."""
-
-        # check there is a page
-        if self.document.getNumberPages() == 0:
-            qt4.QMessageBox.warning(self, _("Error - Veusz"),
-                                    _("No pages to export"))
-            return
-
-        # File types we can export to in the form ([extensions], Name)
-        fd = qt4.QFileDialog(self, _('Export page'))
-        fd.setDirectory( self.dirname_export )
-
-        fd.setFileMode( qt4.QFileDialog.AnyFile )
-        fd.setAcceptMode( qt4.QFileDialog.AcceptSave )
-
-        # Create a mapping between a format string and extensions
-        filtertoext = {}
-        # convert extensions to filter
-        exttofilter = {}
-        filters = []
-        # a list of extensions which are allowed
-        validextns = []
-        formats = document.Export.formats
-        for extns, name in formats:
-            extensions = " ".join(["*." + item for item in extns])
-            # join eveything together to make a filter string
-            filterstr = '%s (%s)' % (name, extensions)
-            filtertoext[filterstr] = extns
-            for e in extns:
-                exttofilter[e] = filterstr
-            filters.append(filterstr)
-            validextns += extns
-        fd.setNameFilters(filters)
-
-        # restore last format if possible
-        try:
-            filt = setdb['export_lastformat']
-            fd.selectNameFilter(filt)
-            extn = formats[filters.index(filt)][0][0]
-        except (KeyError, IndexError, ValueError):
-            extn = 'pdf'
-            fd.selectNameFilter( exttofilter[extn] )
-
-        if self.filename:
-            # try to convert current filename to export name
-            filename = os.path.basename(self.filename)
-            filename = os.path.splitext(filename)[0] + '.' + extn
-            fd.selectFile(filename)
-
-        if fd.exec_() == qt4.QDialog.Accepted:
-            # save directory for next time
-            self.dirname_export = fd.directory().absolutePath()
-
-            filterused = str(fd.selectedFilter())
-            setdb['export_lastformat'] = filterused
-
-            chosenextns = filtertoext[filterused]
-
-            # show busy cursor
-            qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
-
-            filename = fd.selectedFiles()[0]
-
-            # Add a default extension if one isn't supplied
-            # this is the extension without the dot
-            ext = os.path.splitext(filename)[1][1:]
-            if (ext not in validextns) and (ext not in chosenextns):
-                filename += "." + chosenextns[0]
-
-            export = document.Export(
-                self.document,
-                filename,
-                self.plot.getPageNumber(),
-                bitmapdpi=setdb['export_DPI'],
-                pdfdpi=setdb['export_DPI_PDF'],
-                antialias=setdb['export_antialias'],
-                color=setdb['export_color'],
-                quality=setdb['export_quality'],
-                backcolor=setdb['export_background'],
-                svgtextastext=setdb['export_SVG_text_as_text'],
-                )
-
-            try:
-                export.export()
-            except (RuntimeError, EnvironmentError) as e:
-                if isinstance(e, EnvironmentError):
-                    msg = cstrerror(e)
-                else:
-                    msg = cstr(e)
-
-                qt4.QApplication.restoreOverrideCursor()
-                qt4.QMessageBox.critical(
-                    self, _("Error - Veusz"),
-                    _("Error exporting to file '%s'\n\n%s") %
-                    (filename, msg))
-            else:
-                qt4.QApplication.restoreOverrideCursor()
+        from ..dialogs.export import ExportDialog
+        dialog = ExportDialog(self, self.document, self.filename)
+        self.showDialog(dialog)
+        return dialog
 
     def slotFilePrint(self):
         """Print the document."""
@@ -1312,10 +1207,9 @@ class MainWindow(qt4.QMainWindow):
 
         if values:
             # construct comma separated text representing axis values
-            valitems = []
-            for name, val in citems(values):
-                valitems.append('%s=%#.4g' % (name, val))
-            valitems.sort()
+            valitems = [
+                '%s=%#.4g' % (name, values[name])
+                for name in sorted(values) ]
             self.axisvalueslabel.setText(', '.join(valitems))
         else:
             self.axisvalueslabel.setText(_('No position'))
@@ -1351,7 +1245,7 @@ class MainWindow(qt4.QMainWindow):
         ytext = fmt(yv, yt)
 
         t = '%s: %s%s = %s, %s%s = %s' % (
-                info.widget.name, xn, ix, xtext, yn, ix, ytext)
+            info.widget.name, xn, ix, xtext, yn, ix, ytext)
         self.pickerlabel.setText(t)
         if setdb['picker_to_console']:
             self.console.appendOutput(t + "\n", 'error')
@@ -1364,6 +1258,6 @@ class MainWindow(qt4.QMainWindow):
 
     def slotAllowedImportsDoc(self, module, names):
         """Are allowed imports?"""
-
+        from ..dialogs.safetyimport import SafetyImportDialog
         d = SafetyImportDialog(self, module, names)
         d.exec_()
