@@ -306,7 +306,7 @@ class PropertyList(qt4.QWidget):
 
         self.layout = qt4.QGridLayout(self)
         self.layout.setSpacing( self.layout.spacing()//2 )
-        self.layout.setMargin(4)
+        self.layout.setContentsMargins(4,4,4,4)
         
         self.childlist = []
         self.setncntrls = {}     # map setting name to controls
@@ -331,8 +331,9 @@ class PropertyList(qt4.QWidget):
 
             button = qt4.QPushButton(text)
             button.setToolTip(action.descr)
-            button.clicked[()].connect(
-                lambda a=action: setnsproxy.onAction(a, self.getConsole()))
+            button.clicked.connect(
+                lambda checked=True, a=action:
+                setnsproxy.onAction(a, self.getConsole()))
 
             self.layout.addWidget(button, row, 1)
             self.childlist.append(button)
@@ -580,7 +581,7 @@ class TabbedFormatting(qt4.QTabWidget):
 
         # layout for tab widget
         layout = qt4.QVBoxLayout()
-        layout.setMargin(2)
+        layout.setContentsMargins(2,2,2,2)
         layout.addWidget(scroll)
 
         # finally use layout containing items for tab
@@ -831,6 +832,9 @@ class TreeEditDock(qt4.QDockWidget):
     def _constructToolbarMenu(self):
         """Add items to edit/add graph toolbar and menu."""
 
+        def slotklass(klass):
+            return lambda: self.slotMakeWidgetButton(klass)
+
         iconsize = setting.settingdb['toolbar_size']
         self.addtoolbar.setIconSize( qt4.QSize(iconsize, iconsize) )
         self.edittoolbar.setIconSize( qt4.QSize(iconsize, iconsize) )
@@ -847,9 +851,7 @@ class TreeEditDock(qt4.QDockWidget):
                            'nonorthpoint', 'nonorthfunc'):
 
             wc = document.thefactory.getWidgetClass(widgettype)
-            def slotfn(klass=wc):
-                return lambda: self.slotMakeWidgetButton(klass)
-            slot = slotfn(wc)
+            slot = slotklass(wc)
             self.addslots[wc] = slot
 
             actionname = 'add.' + widgettype
@@ -1134,7 +1136,8 @@ class TreeEditDock(qt4.QDockWidget):
         hideshow is True for hiding, False for showing
         """
         ops = [ document.OperationSettingSet(w.settings.get('hide'), hideshow)
-                for w in widgets ]
+                for w in widgets
+                if 'hide' in w.settings ]
         descr = ('show', 'hide')[hideshow]
         self.document.applyOperation(
             document.OperationMultiple(ops, descr=descr))
@@ -1229,7 +1232,7 @@ class SettingLabel(qt4.QWidget):
         self.setnsproxy = setnsproxy
 
         self.layout = qt4.QHBoxLayout(self)
-        self.layout.setMargin(2)
+        self.layout.setContentsMargins(2,2,2,2)
 
         if setting.usertext:
             text = setting.usertext
