@@ -20,6 +20,7 @@
 from __future__ import division
 import os.path
 
+import sip
 from .. import qtall as qt4
 from .. import utils
 from .. import setting
@@ -29,6 +30,9 @@ def _(text, disambiguation=None, context="Tutorial"):
     return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class TutorialStep(qt4.QObject):
+
+    nextStep = qt4.pyqtSignal()
+
     def __init__(self, text, mainwin,
                  nextstep=None, flash=None,
                  disablenext=False,
@@ -60,8 +64,7 @@ class TutorialStep(qt4.QObject):
 
         self.nextonselected = nextonselected
         if nextonselected is not None:
-            self.connect(mainwin.treeedit, qt4.SIGNAL('widgetsSelected'),
-                         self.slotWidgetsSelected)
+            mainwin.treeedit.widgetsSelected.connect(self.slotWidgetsSelected)
 
     def slotNextSetting(self, *args):
         """Check setting to emit next."""
@@ -69,14 +72,14 @@ class TutorialStep(qt4.QObject):
             setn = self.mainwin.document.basewidget.prefLookup(
                 self.nextonsetting[0]).get()
             if self.nextonsetting[1](setn):
-                self.emit( qt4.SIGNAL('nextStep') )
+                self.nextStep.emit()
         except ValueError:
             pass
 
     def slotWidgetsSelected(self, widgets, *args):
         """Go to next page if widget selected."""
         if len(widgets) == 1 and widgets[0].name == self.nextonselected:
-            self.emit( qt4.SIGNAL('nextStep') )
+            self.nextStep.emit()
 
 ##########################
 ## Introduction to widgets
@@ -116,7 +119,7 @@ class StepWidgets2(TutorialStep):
     def __init__(self, mainwin):
         TutorialStep.__init__(
             self, _('''
-<p>Widget can often be placed inside each other. For instance, a graph
+<p>Widgets can often be placed inside each other. For instance, a graph
 widget is placed in a page widget or a grid widget. Plotting widgets
 are placed in graph widget.</p>
 
@@ -278,12 +281,11 @@ the flashing Line tab (%s).</p>
             disablenext=True,
             nextstep=FunctionLineFormatting)
 
-        self.connect(tb, qt4.SIGNAL('currentChanged(int)'),
-                     self.slotCurrentChanged)
+        tb.currentChanged[int].connect(self.slotCurrentChanged)
 
     def slotCurrentChanged(self, idx):
         if idx == 1:
-            self.emit( qt4.SIGNAL('nextStep') )
+            self.nextStep.emit()
 
 class FunctionLineFormatting(TutorialStep):
     def __init__(self, mainwin):
@@ -342,19 +344,20 @@ class DataImport(TutorialStep):
             if k in setting.settingdb:
                 del setting.settingdb[k]
 
-        self.connect(mainwin, qt4.SIGNAL('dialogShown'), self.slotDialogShown )
+        mainwin.dialogShown.connect(self.slotDialogShown)
 
     def slotDialogShown(self, dialog):
         """Called when a dialog is opened in the main window."""
         from ..dialogs.importdialog import ImportDialog
         if isinstance(dialog, ImportDialog):
             # make life easy by sticking in filename
+            dialog.slotReset()
             dialog.filenameedit.setText(
                 os.path.join(utils.exampleDirectory, 'tutorialdata.csv'))
             # and choosing tab
             dialog.guessImportTab()
             # get rid of existing values
-            self.emit( qt4.SIGNAL('nextStep') )
+            self.nextStep.emit()
 
 class DataImportDialog(TutorialStep):
     def __init__(self, mainwin):
@@ -389,7 +392,7 @@ to the original file.</p>
 
     def slotDocModified(self):
         if 'alpha' in self.mainwin.document.data:
-            self.emit( qt4.SIGNAL('nextStep') )
+            self.nextStep.emit()
 
 class DataImportDialog3(TutorialStep):
     def __init__(self, mainwin):
@@ -405,8 +408,7 @@ box or reopen it later.</p>
             nextstep=DataImportDialog4)
 
         self.timer = qt4.QTimer()
-        self.connect( self.timer, qt4.SIGNAL('timeout()'),
-                      self.slotTimeout )
+        self.timer.timeout.connect(self.slotTimeout)
         self.timer.start(200)
 
     def slotTimeout(self):
@@ -417,7 +419,7 @@ box or reopen it later.</p>
                 closed = False
         if closed:
             # move forward if no import dialog open
-            self.emit( qt4.SIGNAL('nextStep') )
+            self.nextStep.emit()
 
 class DataImportDialog4(TutorialStep):
     def __init__(self, mainwin):
@@ -648,7 +650,7 @@ deletes it from the document.</p>
             d.resolve(d.basewidget, '/page1/graph1')
         except ValueError:
             # success!
-            self.emit( qt4.SIGNAL('nextStep') )
+            self.nextStep.emit()
 
 class AddGrid(TutorialStep):
     def __init__(self, mainwin):
@@ -773,7 +775,7 @@ class TutorialDock(qt4.QDockWidget):
         self.buttonbox = qt4.QDialogButtonBox()
         self.nextb = self.buttonbox.addButton(
             'Next', qt4.QDialogButtonBox.ActionRole)
-        self.connect(self.nextb, qt4.SIGNAL('clicked()'), self.slotNext)
+        self.nextb.clicked.connect(self.slotNext)
 
         l.addWidget(self.buttonbox)
 
@@ -806,7 +808,7 @@ class TutorialDock(qt4.QDockWidget):
         self.step = stepklass(self.mainwin)
 
         # listen to step for next step
-        self.connect(self.step, qt4.SIGNAL('nextStep'), self.slotNext)
+        self.step.nextStep.connect(self.slotNext)
 
         # update text
         self.textedit.setHtml(self.step.text)
@@ -825,19 +827,24 @@ class TutorialDock(qt4.QDockWidget):
         if self.step.closestep:
             closeb = self.buttonbox.addButton(
                 'Close', qt4.QDialogButtonBox.ActionRole)
-            self.connect(closeb, qt4.SIGNAL('clicked()'), self.close)
+            closeb.clicked.connect(self.close)
 
     # work around C/C++ object deleted
     @qt4.pyqtSlot()
     def slotFlashTimeout(self):
         '''Handle flashing of UI components.'''
 
-        if self.flash is not self.oldflash and self.oldflash is not None:
+        # because we're flashing random UI components, the C++ object
+        # might be deleted, so we have to check before doing things to
+        # it: hence the sip.isdeleted
+
+        if ( self.flash is not self.oldflash and self.oldflash is not None
+             and not sip.isdeleted(self.oldflash) ):
             # clear any flashing on previous widget
             self.oldflash.setStyleSheet('')
             self.oldflash = None
 
-        if self.flash:
+        if self.flash is not None and not sip.isdeleted(self.flash):
             # set flash state and toggle variable
             if self.flashon:
                 self.flash.setStyleSheet('background: yellow;')
